#####                  	 SCRIPTS FOR PART 1 

### NOTE: The scripts for these analyses require the installation of
# packages that were not downloaded with the original installation.
# The form of the installation command is 
#         install.packages("package name") 
# For Part 1, install the following packages:
#         "moments", "plotrix", "pwr"
# The package is loaded by the command 
#         library(package name)
# e.g.,  to install: install.packages("moments") 
# and yo load: library(moments)

# Also required are functions that are not in the packages. As described
# in "An Overview of the R Scripts," "Functions.R" can be obtained at
# "my R website": https://udrive.oit.umass.edu/jlmyers/RScripts/.
# In the following scripts, references to "my R website" indicate the
# above website. For further information about accessing and saving scripts
# and data files, read the "Overview of R Scripts" at my R website 


# In addition to scripts and data files, my R website also contains a brief
# introduction to R ("Getting Started with R"). For those who have no or
# little experience with R, this proviodes references to useful websites
# and published material, as well as introductory material.

###########################################################

#####               	Script for Session 2
#####                    Exploring the Data
 
### Summary Statistics
# Start by reading the "Royer_Y.csv" file:
     RoyerY=read.csv(file.choose(),header=T)
# This opens your file manager (e.g., Windows Explorer), allowing you to
# select the file you want, and placing it in the R console as a data frame.
# Then,
     head(RoyerY)      # requests first 6 lines of the RoyerY data frame
     attach(RoyerY)    # can now refer to Royer and Y without $ notation

# NOTE: The alternative to attaching a file is to use notation such as
# RoyerY$Royer to refer to the variable, Royer. Many prefer this 
# because it avoids problems. For example, if we fail to detach RoyerY
# before attaching a different data frame with variables also named
# Royer or Y, the results of analyses of the new data may not be correct.
# Also, when starting a new session, the data frame must be reattached. We have
# used both methods at various points in these scripts.

# NOTE: To see what is attached, type
     search()

# For some statistics, type
     summary(RoyerY)   # gives min, max, quartiles, median, mean
 
## Percentile Scores
      quantile(Royer, probs=c(.25, .75))
# calculates scores at 25th and 75th percentles

## NOTE: R uses y=(1-g)*x_sub_j + g*x_sub_(j+1) to calculate
# percentile scores where the scores are in ascending order; i.e.,
# x_sub_1 is the lowest score), and x_sub_j is the score in the jth 
# position; j = 1+[p(n-1)], g =1+p(n-1)-j, and [ ] indicates 
# the integer floor of a quantity.

## SPSS uses a slightly different definition of quantiles
# Same formula for y but j = [p[n+1]], g = p(n+1)-j
# R can calculate the SPSS formula if you prefer:
	quantile(Royer, probs=c(.25, .75),type=6) 
# see "help(quantile)" for the 9 types available.

## Exercise 2.1 
#(a) Using the Y data and R's default quantile command,
# calculate the 15th and 85th percentile scores.
# Then calculate j and g (using R's definition; see above) and verify that 
# the scores are given by (1-g)*x_sub_j + g*x_sub_(j+1).
# (b) Now use R to calculate the results that SPSS's formula would give.
# Use the quantile function with "type = 6" and verify using SPSS's 
# formula for Y.

# Now let's see what objects are in our workspace, and remove those
# we no longer need,
      ls()
# OR (preferable; see the introductory text in the Functions.R script)
	LS()                                   
# If you've done the exercises, you should have several objects;
# these my be removed. However, retain any functions and RoyerY,
# which we will continue to analyze.

# NOTE:  If following the rm command,you type LS(), the list of objects
#  should have only functions and the one data frame, RoyerY.
 

## More Statistics
      sapply(RoyerY,sd)           #Standard deviations of the two variables
      var(RoyerY) 		#Diagonal elements are covariances
#If we only want the variances, we can use indexing of RoyerY; e.g.,
      c(var(RoyerY[,1]),var(RoyerY[,2])) #"c" concatenates the two variances							#variances
#or
      sapply(RoyerY,sd)^2        #squared std. deviation

# To get skewness and kurtosis, and related tests, the "moments"
# package should have been installed. Then,
      library("moments") 	#loads functions to get skewness, kurtosis
      skewness(RoyerY)
      kurtosis(RoyerY)

#To test skewness:
       agostino.test(Royer, alternative ="two.sided")
#"greater", "less" are other alternatives -- for example,
      agostino.test(Royer, alternative = "greater")
# The p-value indicates significant negative skew (long left tail)

# To test kurtosis:
      anscombe.test(Royer)
# default is "two-sided; other alternatives are "greater", "less"

## NOTE: R and SPSS calculate different skew and kurtosis statistics.
# R calculates Pearson's b statistics whereas
# SPSS calculates Fisher's g statistics.

## Exercise 2.2
#  Test skewness and kurtosis of the Y data. What do you conclude?

## Exercise 2.3 
# Combine the following two data sets into a single data frame, "dataset,"
# with two columns containing x1 and x2.
# x1= 66 62 54 48 46 44 38 29 18 15
# x2 = 80 72 66 64 62 50 42 36 30 28
# Using R, find the summary statistics for dataset and describe the results.
 
### Graphing Data
# Is RoyerY still in the workspace? Use "ls()" or "LS()" to check.
# Is it still attached? Use "search()" to check.
# The next few commands assume RoyerY is attached.
# If it isn't, "attach(RoyerY)"

## Histograms
# Many of the arguments listed in the help output are optional. For
# example, try
      hist(Royer)         #histogram of the Royer data
# To turn the plot off, either click the "x" in the upper right, or
      dev.off()

# We have some sense of the distribution. But we can do much better with
      hist(Royer, breaks=20, main="Royer Addition Scores",
      xlab="Addition Scores", ylab="Frequency", col = "green")
# "breaks" refers to the number of bins, "main" provides the title of the
# graph, and "xlab" and "ylab" provide the labels for the X and Y axes.
# We can include color by adding 'col = "green" ' or some other color.
# colors() produces over 650 possible color names and the website
# "http://research.stowers-institute.org/efg/R/Color/Chart/"
# shows the colors corresponding to each number.

 # To save the graph in a jpeg file:
      jpeg("royer.jpeg")
      hist(Royer, breaks=20, main="Royer Addition Scores",
      xlab="Addition Scores", ylab="Frequency", col = "green")
      dev.off()  		#Saves the jpeg file in your default directory
 
# To copy in order to paste in a manuscript
# Right click on plot and select "copy as metafile", then
# "dev.off()" to close plot

# To print the graph:
      win.print(width = 7, height = 7, pointsize = 12, printer = "",
      restoreConsole = TRUE)
      hist(Royer, breaks=20, main="Royer Addition Scores",
      xlab="Addition Scores", ylab="Frequency")
      dev.off()
# This prints the histogram and returned control to the screen null device

##Exercise 2.4
# Print a histogram of the Y data. If your printer permits color,
# include an instruction for a color of your choice.
# Before printing, experiment with different colors,
# labels, number of bins font size. Check other options by using
# the instruction "?hist"; also "?par" is extremely useful.
 
# To plot our Royer and Y histograms side by side
      op = par(mfrow=c(1,2)) #One row with two panels
      hist(Royer)
      hist(Y)
      op                 	#Return to 1 panel
      dev.off()

# Another data set -- get "Royer rt_speed.csv"
      detach(RoyerY)      # So there won't be confusion of variable labels
      mult_time = read.csv(file.choose(),header=T)   	#multiplication data    
      m78 = subset(mult_time, gender=="Male"& grade>6)
# Get rows that are male and in grades 7 and 8 from mult_time data frame
      m78  #data frame with RT and speed scores for male 7th & 8th graders

# Suppose we had wanted only the RT data for males in grades 7 and 8:
      m78rt = subset(mult_time, gender=="Male"& grade>6, select=M_RT)
      m78rt	   # "subset" selects the rows and "select" gets the column(s)

# You may wish to retain the gender and grade information. One way:
      M78rt=subset(mult_time, gender=="Male"& grade>6, 
		select=c(gender,grade,M_RT))
# Note that m78rt and M78rt are different objects

## Exercise 2.5
# (a) Extract the 4 subsets of data, male and female, RT and speed
# (b)Plot the histograms for these data sets in 4 panels of a single display,
#  each with a title, and labeled axes.

## Boxplots
# A simple boxplot
      boxplot(RoyerY$Royer, main="Addition Scores", ylab="Percent
		Correct")
      dev.off()
 
# Two boxplots side by side with color and notches
      boxplot(mult_time$M_RT~mult_time$gender, ylab="Response Time",
      notch=TRUE,main="Royer Multiplication Response Times", col = "yellow")
#"M_RT~gender" means M_RT "as a function of" gender;
# "notch=TRUE" asks for notches at the median; if the notches don*t overlap,
# it is likely that the medians differ significantly. In this case,
# the notches overlap, providing no evidence of a difference
# between the population medians. However, R issued a warning.
#If we view the width of the notch opening as a rough confidence interval
# about the median, the warning indicates that the lower bound
# for the Male median is below the hinge.
           dev.off()          #Close the boxplot figure

## Exercise 2.6 
# Continuing with the same data set, construct a plot containing two panels;
# one showing boxplots with RT as a function of gender,
# the other with boxplotds of the speed data as a function of gender.
# Compare the two boxplots with respect to the distributions of scores.

## Stem-and-Leaf, and Summary Statistics
      attach(RoyerY)
      stem(Royer) #stem and leaf
      stem(Royer,scale=2) #twice as many stems
      summary(Royer)      #min, 1st quartile, median, mean, 3rd quartile, max
      fivenum(Royer)      #min,lower hinge, median, upper hinge, max

#The fivenum results use Tukey's calculations for median and hinges
# They tell us that any score less than 79-(1.5)(94.5-79)
# is an outlier. Therefore, the 47 and two 50's are outliers by Tukey's
# definition.

##Exercise 2.7 (a) Get stem-and-leaf plots for the M_RT data
# in the mult_time data frame. Try various scales and select one
# you think provides the best picture of the data.
#(b) Get the 5-point summary. What is the cutoff for outliers?
#(c) You will need the "moments" package for this.
# If you have not previously installed and loaded it, do so now.
# Get R*s skewness and kurtosis statistics for the M_RT data,
# and perform the significance tests.
# Are the results consistent with the conclusions
# you can draw from your answers to parts (a) and (b)?
#(d) Repeat parts (a) - (c), this time using the M_Speed scores.
# Using the (a) - (c) results, what are the effects of the transformation?

 
## Kernel Density Plots
# Histograms, boxplots, and stem-and-leaf plots provide different views
# of the sample distribution. In many cases, it is useful to have a sense
# of the distribution of the population from which the sample was drawn.
# The following commands provide different estimates of the population
# for the Royer addition accuracy scores.
# First attach the data frame you want to plot. Then,
      op=par(mfrow=c(2,2)) #Open a four-panel plot, 2 rows and 2 columns
      plot(density(Royer)) #Plot with the default bandwidth (bw)
      plot(density(Royer, bw = 2))
      plot(density(Royer, bw = 8))
      plot(density(Royer, bw = 12))
 
# Now return to a single panel and close the figure
          op         
          dev.off()  

## Exercise 2.8 
# Plot the kernel densities for the m78 RT and m78 speed data
# in two panels side by side. Use the default bandwidth. Then.
# use one half as large and one twice as large. Discuss the results

## q-q Plots
#The default in R*s qqnorm function is to plot observed values as a function
# of theoretical values.
# We can  plot theoretical as a function of observed values (as SPSS does)
# by specifying "datax = TRUE." Once we construct our plot, a second function,
# "qqline" adds a line that passes through the first and third quartiles;
# the line is not added when *datax = TRUE" is included in the statement.
      qqnorm(m78$M_RT,main = "Royer Multiplication Response Times")
      qqline(m78$M_RT)
# Close the plot
      dev.off()

## Exercise 2.9 Plot the q-q plots for the m78 RT and m78 speed data in two
# panels side by side. Plot the female data in two plots below them.
# Do the data appear to be normally distributed?

#House cleaning. Use ls() to see objects in workspace and search()
#    to see what is attached
          search()
          detach(RoyerY)
          ls()		# or LS() if you have the function in your workspace
          rm(RoyerY,m78,m78rt,M78rt,op)


### Tabling Means and Standard Errors. 
# Read in "Beck_D.csv"
      Beck_D= na.omit(read.csv(file.choose(),header=T))
	 names(Beck_D)	# seasonal depression scores as a function of sex
				# and marital status
# "na.omit" drops cases with missing scores
	 attach(Beck_D)
      d_means=rowMeans(Beck_D[,3:6]) 
# d_means are means over the 4 seasons (cols.3 to 6)for each subject 
      group.means=aggregate(d_means,list(mstatus, sex), mean)
      colnames(group.means)=c("mstatus","sex","d_mean") #Assign column names
      group.means 	# mean of subject d_means as a function
				# of marital status and sex.    
# We now have the depression means (averages over seasons) as a
# function of marital status and sex. We also want the standard errors
 
#You should have installed the plotrix package. Then
      library(plotrix)
      group.ses=aggregate(d_means,list(mstatus, sex),
          std.error,na.rm=TRUE)   	
      colnames(group.ses)=c("mstatus","sex","d_se")
      group.ses		# std. error of subject d_means as a function
					# of marital status and sex.
# Because there is only 1 male score in the mstatus=4 category, we 
# see "NA" (not available) for the standard error.

      beck_d.groups = cbind(group.means$mstatus,group.means$sex,
               group.means$d_mean, group.ses$d_se)
      colnames(beck_d.groups)=c("mstatus","sex","d_mean","d_se")
      beck_d.groups
# Note that there are no women in mstatus=4. This will cause a problem
# when we try to plot the data. To avoid this:
	mstatus4 = cbind(4,1,0,0)
	nugroups=rbind(beck_d.groups[1:9,],mstatus4,beck_d.groups[10:11,])
	nugroups			#Compare with beck_d.groups

## Exercise 2.10 
# Tabulate the means and standard errors for the Royer M_RT scores
# (mult-time data frame if saved or recreate from Royer rt_speed.csv) as a 
# function of grade and gender.


### Plotting Means and Standard Errors.
## Bar Graphs
# An internet search for "R bar plots" ("R error bars" is also helpful)
# reveals several approaches to obtaining bar plots with standard error
# or confidence interval bars. We begin by tabling the means and se?s:
      yy = matrix(nugroups[,3],6,2,byrow=FALSE)
# This command yields a 6 x 2 matrix of means.
# "byrow=FALSE" means that the columns are filled first.
      ee = matrix(nugroups[,4],6,2,byrow=FALSE)
# This command yields a 6 x 2 matrix of std. errors. 
# Then,
      barx=barplot(yy,beside=TRUE,ylim=c(0,16),names.arg=c("Men","Women"),
      axis.lty=1,xlab="Sex",ylab="Beck Depression Scores",col=rainbow(6))
      legend("topright", c("single","married","with partner","separated",
      "widowed","divorced"),fill=rainbow(6))
#We next calculate the upper and lower limits
#    and then apply to the figure which must be open at this point.
      LL=yy-ee
      UL=yy+ee
      arrows(barx,yy,barx,UL,angle=90)
      arrows(barx,yy,barx,LL,angle=90)
#NOTE: *beside=TRUE" places the bars next to each other;
# *beside=FALSE" would stack the six bars for each sex.
# If *axis.lty=1" is omitted, the x axis line is not drawn.
# We could have added a title by inserting in the barplot statement
#  e.g. main="Seasons Mean Depression Data".
# We could have changed the font size as indicated earlier
# In the current example, the xlab (*sex") is superfluous
# but we wished to remind you that the x axis can be labeled.

#NOTE: The upper and lower limits (UL, LL) can be defined in several ways;
# e.g., UL = yy+2*ee would provide an approximate upper bound
# for a confidence interval about the mean.

# Let*s close the bar graph and move on.
      dev.off()

#Let's clean up the workspace
	detach(Beck_D)
     rm(barx,nugroups,Beck_D,d_means,ee,LL,group.means,group.ses,
		beck_d.groups,mstatus4,UL,yy)
# Use LS to check; only functions, RoyerRT, and mult_time data frane should 
# still be in the workspace.

## Exercise 2.11r
# You should still have in your workspace the mult_time data frame.
# If you removed it, reconstruct it. Then plot M_RT as a function of 
# grade and gender.

## Line graphs for means with error bars
# The "Royer_rt.csv" # data file contains response times (rt) for grades 3 to 8.
# We will work with the subrt (subtraction response time) data of that file.
      RoyerRT=read.csv(file.choose(),header=T,fill=T)
      attach(RoyerRT)
      srt=subset(RoyerRT, select=c("grade","gender","subrt"))
      head(srt)
      srt.means=na.omit(aggregate(srt$subrt,list(srt$grade, srt$gender),
		mean))
# na.omit removes all lines with missing scores
      colnames(srt.means)=c("grade","gender","mean") #Assign column names
     srt.means
# If not still loaded,
      library(plotrix) #To have std.error function loaded

      srt.ses=na.omit(aggregate(srt$subrt,list(srt$grade,srt$gender),std.error))
      colnames(srt.ses)=c("grade","gender","se")
      sex=c(rep(0,6), rep(1,6))
      srt.groups = cbind(srt.means$grade,sex,srt.means$mean,srt.ses$se)
      colnames(srt.groups)=c("grade","sex","s_mean","s_se")
      srt.groups=data.frame(srt.groups)
      mm = matrix(srt.groups[,3],6,2,byrow=FALSE) #matrix of group means
      ss = matrix(srt.groups[,4],6,2,byrow=FALSE) #matrix of group std errors
      girls=srt.groups[1:6,3]
      boys=srt.groups[7:12,3]
      myplot=plot(girls,type="o",axes=F,ann=F,ylim=c(1.5,5))
# type = "o" asks for both lines and symbols
# axes=F, ann=F ask for no axes or labels
# ylim sets limits on Y axis
      axis(1,at=1:6,lab=3:8)        	#Add x axis with 6 labeled points
# or could have lab=c("3",?4",?5",?6",?7",?8")
      axis(2,at=c(1.5,2,2.5,3,3.5,4)) 	#Add Y axis at specified points
      box() 						#Place the graph within a box
      lines(boys, type="o", pch=22, lty=2)
# pch specifies the symbol and lty the line type
      title(main="Royer Subtraction Data",font.main=3)
# font.main specifies font size
      title(xlab="Grade")           #X-axis legend
      title(ylab="Subtraction RT")      #Y axis legend
      legend("topright",c("girls","boys"), pch=21:22, lty=1:2, cex=.8)
      UB = mm+ss                   	#upper bound=mean + std. error
      LB= mm-ss                         #lower bound
      x=c(1:6)
      arrows(x,mm,x,UB,angle=90)
      arrows(x,mm,x,LB,angle=90)

# Close the plot 
      dev.off()
 
##Exercise 2.12 
# Because the two curves are so close to each other,
# the plot is very messy when the std. error bars are included.
# Plot the curves separately in a row of two panels.

# Cleanup
      detach(RoyerRT)        
	rm(srt,srt.groups,srt.means,srt.ses,sex,ss,boys,girls,myplot,UB,LB,x,mm)

############################################################

#####                          Script for Session 3
#####                     Frequency Tables and Probabilities

### Two-way Tables
## Cell Frequencies
# First create a table of students' grades
      grades = sample(rep(c("A","B","C","D","F"),c(5,20,10,3,2)),40)
# "rep(c("A","B","C","D","F"),c(5,20,10,3,2))" dictates the grades and
# the number of each. The default for "sample" is "replace = FALSE" which
# dictates that sampling is without replacement. "replace = TRUE"
# dictates sampling with replacement. 
      sex=rep(c("boys","girls"),each=20)
      my.class = data.frame(sex,grades)
      head(my.class)
# Now tabulate the letter grades
      class = table(sex,grades); class

## Marginal Frequencies
      margin.table(class)   	#How many observations?
      margin.table(class,1) 	#row mrginal frequencies
      margin.table(class,2) 	#column marginal frequencies

## Joint Probabilities
# First get the proportion in each cell
      class/margin.table(class) 	 #Divide class entries by total N
      sum(class/margin.table(class)) #Do they sum to 1.0?

# A simpler way to obtain the joint probabilities
      prop.table(class)
      sum(prop.table(class))			# As a check

## Conditional Probabilities
# The probability of the grade given sex [p(grade|sex)]
      prop.table(class,1)
      rowSums(prop.table(class,1))		

## Exercise 3.1 
# For the current data set, find the probabilities
# P(sex|grade) and check your result

## Exercise 3.2 
# Assume 4 age groups and three drug outcomes. Let
# age = c(rep("A1",5),rep("A2",3),rep("A3",7),rep("A4",5)),
# outcome = c(rep("Worse",2),rep("Same",10),rep("Better",8)),
# results = data.frame(age,outcome)
# Form a frequency table and find the conditional probabilities 
# of the outcomes given age.


### Three-way Tables 
## Frequencies
# First create the data set
      teacher = rep(rep(c("X","Y"),each =10),2)
      teacher
      nuclass = cbind(my.class,teacher)
      head(nuclass)

# Now create the frequency table
      nuclass.f = table(nuclass)
      nuclass.f 			#This is a frequency table

# Collapse the frequencies into a two-way table;
# for example, to pool frequencies over sex,
      table(grades,teacher)

# We can index levels of the three factors,
# noting that the first position indexes the level of sex (rows),
# the second indexes grades (columns), and the third indexes teacher (blocks).
# For example, to obtain only the 2-way table for boys
      nuclass.f[1,,]  # Note commas in place of indices for factors 2 and 3

#To get frequencies of C grades (3rd column) by sex and teacher
      nuclass.f[,3,]

#Suppose we wanted a new table with grades as "pass" (A through D)
# and "fail" (F). We first count the passes and failures:
      n_f = sum((grades == "F")*1) 
# grades == "F" yields TRUE, else FALSE
# TRUE*1 = 1, else 0, sum is number of failures (n_f)
      n=length(grades)
      nugrade=c(rep("pass",n-n_f),rep("fail",n_f))
      nugrade
# Sort nuclass on "grades":
      class2=nuclass[order(grades),] 	#orders class from A to F
      class3= cbind(class2,nugrade)
      class3						# Now have letter and pass/fail grades
	 table(sex,teacher,nugrade)    	# 3-way frequency table of pass/fail   			

## Three-Way Probabilities
      prop.table(nuclass.f)              #proportion in each cell
      margin.table(nuclass.f,2)/40       #proportion of each letter grade
      prop.table(nuclass.f,3)            #P(sex & grade|teacher)
      prop.table(nuclass.f,c(1,3)) 	 #P(grade|sex & teacher)
								 # 1=sex, 2=grade, 3=teacher

## Exercise 3.3
# From Exercise 3.2, you should have a table of drug outcomes with six subjects
# in each of four age groups. Use the next two commands to add a variable:
# condition0= c(rep("severe",14),rep("moderate",10))
# condition=sample(condition0,24,replace=FALSE) #Sample without replacement to
# permute the 24 occurrences of "severe" and "moderate"
#(a) Add the condition variable to your original table. You should have
# a table with 24 rows and columns for age, outcome, and condition.
#(b) From the table in (a), use the table function to construct
# a 3-way frequency table.
#(c) Using the appropriate R command, construct a table of
# the joint probabilities.
#(d) For each combination of age and outcome,
# we wish to know the proportion of each condition;
# e.g., P(condition=severe|age=A1 & outcome = 2).
#Use the prop.table function to construct the appropriate table.


## House cleaning
	rm(class,class2,class3,grades,my.class,n,n_f,nugrade,nuclass,
                     nuclass.f,sex,teacher)
  
############################################################

#####                  Script for Session 4
#####          The Binomial Distribution and Hypothesis Tests

### Four Basic Functions Applied to the Binomial Distribution
## The dbinom Function
#To get the probability of exactly one success in 4 trials
# with the probability of a success on each trial equal to .25,
# We could apply the usual binomial formula:
      factorial(4)/(factorial(1)*factorial(3))*(.25^1)*(.75^3)
# or we can simply apply R's binomial function for the density:
# 		dbinom(x,size,prob) 
# where x is the number of successes (1 in this case),
# size is the number of trials (4), and prob is the probability of a success
# on a single trial (.25). Then the probability is given by
           dbinom(1,4,.25)

#To get the entire distribution:
      x=(0:4)            		# x contains the numbers from 0 to 4
      round(dbinom(x,4,.25),3)     #"round" answer to 3 decimal places
      sum(dbinom(x,4,.25)) 		# Should sum to 1

#To get a table in which the columns hold the 5 probabilities
#(for x = 0 to 4) for p = .25, .5, and .75, we can use a "while" loop:
      prob=matrix(0,5,3) 		#initialize a 5 x 3 matrix of zeros
      x=(0:4)
      p=c(.25, .5, .75)
      j =1
      while (j <=3) {     #Note brackets {and } to enclose loop
           prob[,j]=dbinom(x,4,p[j])
            j=j+1
      }
      round(prob,3)
#We might wish to label the rows and columns
      rownames(prob)=c("0","1","2","3","4")
      colnames(prob)=c(".25",".50",".75")
      round(prob,3)
      colSums(prob) #each column should sum to 1

## A function, "dbin_table", to generate tables for any x and p,
# is in the Functions.R script at my R website. Test the function with
      x=(0:4)				# possible values of x
      size=4				# number of trials
      nr=5				# number of possible outcomes (size+1)
      p=c(.25,.5,.75)
      dbin_table(x,size,nr,p)
#OR just enter the values into the funct:
      dbin_table((0:4),4, 5, c(.25,.5,.75))

## Exercise 4.1
# Find the probability of 3 or more correct for the values in the last
# statement. We want three values, one for each of the three p values.

## The pbinom function
# Exercise 4.1 suggests finding cumulative probabilities by using the
# dbinom function, and summing. R provides a siumpler alternative,
# pbinom(q, size, prob) where q is the quantile.
# For example, if we want the probability of
# 3 or less successes in 10 trials when the probability of a success
# on a single trial is .25:
     pbinom(3,10,.25)
#Probability of 4 or more (the upper tail) is
      1-pbinom(3,10,.25)
# OR
      pbinom(3,10,.25, F) #The F is shorthand for "lower.tail = FALSE.?

#We can generate all the cumulative probabilities using pbinom:
      round(pbinom((0:10), 10, .25), 3)
# In this case, .056 = p(0); .244 = p(0 or 1); .5256 = p(0 or 1 or 2),etc.
# If we want the upper tail probability:
      round(pbinom((0:10),10,.25, F),3)
#.944 = P(more than 0), .756 = P(more than 1), etc.

## Exercise 4.2
# Use the pbinom function to verify your answer to Exercise 4.1.
# You may do it in three steps, one for each of the three prob values,
# or you can try to write a while loop or even a general function
# to do the calculations.

## The qbinom Function
# qbinom gives a value of q corresponding to a cumulative probability.
# For example, if we want the value of q that has 30% of the distribution
# below it, still assuming 10 trials and prob = .25, the function is
      qbinom(.3,10,.25)

# NOTE: The answer, q = 2, does not exceed exactly 30% of the distribution
# because the distribution is discrete. P(0,1,2) = 
	pbinom(2,10,.25)
# which equals .526. q (2 in this example) is the smallest value such that
# the probability is p (.3 in the example) or greater of a success.

# We are more often concerned with the upper tail. For example,
# if we want the 5% cutoff in the upper tail for 20 trials, with
# the probability of a success on a trial = .5, we have
      qbinom(.05, 20, .5,FALSE)
#The answer,q= 14, is the largest value such that p(q or more) <= .05.
# If 13 were included in the critical region, the probability would 
#    exceed .05.

## Exercise 4.3
#(a) Assuming 20 trials and p(success) = .5 on a single trial,
# what is the largest value of q that is exceeded 10% or less of the time?
#(b) Again assume 20 trials and a success probability of .5 on each.
# we might conclude that the coin was biased if there were less
# than a 5% chance of obtaining headsa nd less than a 5% chance
# of obtaining tails. If q is the number of heads, what are the two cutoffs?
#(c) What are the exact probabilities of falling in the upper and lower tails
# of the distribution, assuming the tails are defined by the two values
# of q in your answer to (b)?

## Exercise 4.4
# (a) Using the barplot function (see Session 2), plot the densities
# for the 20-trial experiment. Note: if you wish to label the X axis,
# first define a variable with the values of x,
# the numbers of possible successes; for example,
# xnames=c(*1","2","3", ....,"20"). Then a parameter of the barplot function
# is names.arg=xnames (see ?barplot).
# (b) Plot the cumuolative probabilities.

## The rbinom Function.
#The rbinom function generates random sets of numbers of successes
# and has the general form rbinom(n,size,prob)
# where n is the number of samples,
# size is the number of obnservations in each sample,
# and prob is the P(success on a single observation). For example,
      set.seed(130320)    #Sets the seed for the random number generator
      rbinom(10,5,.5)
 
# My result (yours will differ if you use a different seed)is
#    [1] 3 4 3 4 2 3 4 4 2 3
# indicating that there are 3 successes in the first sample of 5 trials,
# 4 in the second sample, etc.
 
# NOTE: The four functions we have considered are similar to those for 
# other distributions. We will consider those for the normal and t
# distributions in Sessions 5 and 6.

## Exercise 4.5
#(a) Using dbinom, what is the probability density distribution
# of numbers of heads for an unbiased coin in a 4-trial experiment?
#(b) Draw 1024 samples of size 4 using the rbinomial function.
# Call the result bindata, Then tabulate the distribuion and compare
# with your answer to (a). To tabulate, use "round(table(bindata)/1024,3)".
 

### Hypothesis Tests
# Suppose we wish to test the hypothesis that the probability of a success
# on each of 20 independent trials is pi = .25. We will test the null
# hypothesis that pi=.25 against the alternative that pi > .25 with
# alpha = .05. The qbinom function as the basis for our decision rule:
      qbinom(.05, 20, .25, F)
#The result says that under the stated conditions, we reject
# the null hypothesis that pi = .25 in favor of the alternative
# that pi > .25 if there is a success on 8 or more of the 20 trials.
# The actual value of P(x>8|pi=.25) is .041. given by the pbinom function:
      pbinom(8,20,.25, F)	# q = 8
# Note that a criterion of 7 successes yields a p_value > .05:
      pbinom(7,20,.25, F)

# To plot P(Type 1 error) as a function of the critical value,
# the commands are:
      crit = (0:20) 		#all possible critical values from 0 to 20
      p_values= pbinom(crit,20,.25, F) 	#alpha levels correcsponding to
								# the critical values
      plot(p_values~crit, type="l", yaxp=c(0,1,20), xaxp=c(0,20,20))
# Type ?par or help(par) for information about yaxp, xaxp, and
# other graphic parameters

#Turn off the plot
          dev.off()


### Power Functions
# We create a power function tabulating and then plotting
#    P(reject H0|pi > .25). First create a vector of the alternative 
#    values of pi we might hypothesize:
          alt_pi= seq(.25,1,.05) 		# .25, .30, .35, ..., 1
# Recall that our decision is to reject H0 if there are 8 or more successes
# in 20 trials. Therefore
      power=round(pbinom(8,20,alt_pi,F),3)
#Power is the probability of 8 or more successes in 20 trials given that
# the probability of a success is alt_pi.
      power

#Suppose we had 40 trials. Now our critical value is 15:
      qbinom(.05,40,.25,F)
#Therefore,
      power2 = pbinom(15,40,alt_pi,F)
#To view the two power functions to see what doubling N has done:
      plot(alt_pi, power,type="o", xlab="PI",ylab="POWER",
      cex.axis=1.5, cex.lab=1.5,xaxp=c(0,1,10))
# Now plot the Power2 curve
      points(alt_pi,power2,type = "o",pch=22,col = "red")
      legend("bottomright",c("n=20","n=40"), pch=21:22,, cex=.8)

# Turn off the plot
           dev.off()

## Exercise 4.6
#(a) Suppose alpha = .10. Calculate the power function assuming n = 20.
#(b) Plot the power functions for alpha = .05 and alpha = .10.
 
# Clean up
           rm(alt_pi,crit,j,nr,p,p_values,power,power2,prob,size,x)

############################################################

#####   	   Script for Session 5
##### 	  The Normal Distribution

### Sampling Distributions
## Distribution of a Discrete Random Variable
# In Figure 5.1(b) we plotted the sampling distribution of the mean
# based on samples obtained by tossing two dice. We replicate that
# sampling experiment with 5000 trials, each consisting of a toss
# of a pair of dice:
      die=(1:6)
      tosses=sample(die,10000,replace=T)        # 10,000 indpendent coin tosses
      mean.toss=rowMeans(matrix(tosses,5000,2)) # Arrange tosses in a 5000 x 2
									   # matrix; get the row means
      plot(table(mean.toss),ylab="Frequency")   # The plot of 5000 means of 2 
									   # tosses is symmetric
# Turn off the plot
               dev.off()

## Exercise 5.1
# Assume that each of 5000 samples involves 10 tosses.
# Plot the sampling distribution of the mean.
# What are the mean and standard deviation of your sampling distribution?
# What should they be? (differences from your results should be slight
# and due to sampling error)

## Central Limit Theorm
# The central limit theorem is nicely illustrated by drawing large samples
# from a population that has a skewed distribution such as the exponential.
# The density function smooths the fit (see Session 2). The following command
# shows how the exponential looks:
       plot(density(rexp(5000)))
# This command randomly samples 5000 scores from the exponential population
       dev.off()
# Now take 5000 samples of size 10 and plot the means
      exp.means=rowMeans(matrix(sample(rexp(50000),replace=T),5000,10))
       plot(density(exp.means))
# Note the approach to normality although there is still some skewness.
# Turn the plot off
	  dev.off()

#Let's clean up 
      rm(die,exp.means,mean.toss,tosses)
 
##Exercise 5.2
# To evaluate the approach to normality with increasing N,
# plot the q-q function for the sampling istribution of the mean
# based on 1000 samples of size 2 and size 5 from the exponential 
# distribution. Also get measures of skewness and kurtosis.
# See Session 2 for the appropriate R functions.

##Exercise 5.3
# For the samples drawn in Exercise 5.2, what are the means and
# standard deviations of the two sampling distributions? How do
# they compare with the theoretical values for samples of this size?
 

### The Normal Distribution
## The dnorm Function
# This gives the density (basically the height), f(x), of the normal curve
# at a point (or set of points). For example, f(0) for the standard
# (i.e., mean = 0, sd =1) normal distribution is given by
      dnorm(0)
# and to find f(x) at several points
      dnorm(c(-1,0,1))

#We could also have other means and standard deviations:
      dnorm(110,mean=100,sd=15)
# OR
      dnorm(110,100,15)
# Of more interest are probabilities, which we consider next

## The pnorm Function
# The general form is pnorm(x, mean,s d, lower tail)
# If lower.tail is omitted, the default is TRUE; R return P(X < x)
# If lower.tail = FALSe, R returns P(X >x)
# Consider the following examples:
      pnorm(130, 100,15)                     #P(X <130, mu=100, sigma=15)
      pnorm(130, 100,15,F)                   #P(X >130, mu=100, sigma=15)

## The qnorm Function.
# This function has the general form 'qnorm(p, mean,s d, lower tail)'.
# For example, suppose we again have mean=100 and sd=15, and we want to know
# the score that exceeds .977 of the population:
      qnorm(.977, 100, 15)		# = 129.9309
# The score exceeded by .023 of the population is
      qnorm(.023, 100, 15)		# = 70.0691
# Note that the preceding two results are equidistant from the mean (100)
# because the normal distribution is symmetric.
 
## Exercise 5.4
# In a test of reading ability, the population mean is 72 and the sd
# is 8. A class of 16 in one school has a mean score on the test of 76.
#(a) Using R, calculate the .95 confidence interval for the population mean. 
# (b) Based on your answer to (a), Is the class average significantly better
# than the population average? Assume a two-tailed alternative hypothesis
# with alpha = .05.
 
## The rnorm Function
# This function generates random samples from normally distributed
# populations and has the general form rnorm(n,mean,sd). For example:
      rnorm(20,75,5)           #Sample 20 scores from N(mu=75,sigma=5)


### Inferences about Means
## Exploring the Data
# We usually use the t distribution but for large samples, the normal will
# give similar results.  Read the total cholesterol (TC) data for men over 50
# ("TC over 50.csv").
      tc.df= read.csv(file.choose(),header=T)
      head(tc.df) 

# The tc_mean column will be the dependent variable. The dependent variables
# are described in the Seasons description file that can be downloaded
# from my R website.
      dim(tc.df) 		# How many subects? Variables?
      attach(tc.df) 	# to facilitate referring to variables
      summary(tc_mean) 	# Get some basic statistics
      boxplot(tc_mean) 	# A look at the data. Note the one outlier.
# Now turn off the plot
           dev.off()

# Which score is the outlier? This helps to answer:
      mean.tc=sort(tc_mean)        #Sort from low to high
      head(mean.tc)                          
# 104.75 is clearly the outlier

# The function, sort.data.frame, is very useful. Sorts the entire table
# with reference to a designated column. It may be found under Session 5
# in the Functions.R script at my R website. Originally obtained from
#   http://snippets.dzone.com/posts/show/470) 

# As an example, sort tc.df with reference to the tc_mean column
# NOTE: x is a data frame, and key is a column.
      tc2.df = sort.data.frame(tc.df,key="tc_mean")
#Now delete the outlying subject
      nu_tc = tc2.df[2:117,]   #nu_tc excludes the outlying row
      head(nu_tc)              #subject with 104.75 is gone

# To save the new table on my PC in my Data Sets folder:
      write.table(nu_tc,file="C:/R_Stats/DataSets_csv/TC_Data_2 over 50.csv",
      col.names=T,row.names=T, sep = ",")

## Exercise 5.5
# To further explore the data, plot the Q-Q function for the tc_mean data
# (nu_tc table). Also get and test the skewness and kurtosis values
# with and without the outlying case.
 

### Confidence Intervals and Normal Probability (z) Tests
      ybar=mean(nu_tc$tc_mean)
      library(plotrix) 		#Should have installed plotrix previously
      sem =std.error(nu_tc$tc_mean)
      z_crit = qnorm(.975,F)        #default: mu=0,sigma=1
      LB = ybar-z_crit*sem          
      UB = ybar+z_crit*sem
      LB; UB                        #lower and upper bounds of CI =
							 #  220.3769 and 231.0635
# The z test confirms that the mean tc level is significantly greater than 200
      z=(ybar-200)/sem; z
 
## Exercise 5.6
# Create a new variable, diff12, the difference between the means of
# tc1 and tc2. Using R, construct a confidence interval for that difference.
# What do you conclude about whether tc levels change from winter to spring?
 

## Power of the z Test
# Assuming that alpha =.05, sigma= 3.897, the planned study has 100 subjects,
# and the alternative hypothesis is HA:mu = .557,
# what is the power of our test?
      z_crit=qnorm(.95,F);z_crit #z needed to reject H0 (one-tailed)
      z_mean=(.557-0)/.3897; z_mean #transfroming mean tc to z score
      power =pnorm(z_crit-z_mean,0,1, F); power

# Let's cleanup
      detach(tc.df) 
  	 rm(LB,mean.tc,power,sem,tc.df,tc2.df,UB,ybar,z,z_crit,z_mean, nu_tc)
 
############################################################

#####                     Script for Sessions 6/7
#####               The t Distribution and its Applications

# NOTE: This session uses a power function in the "pwr" package.
# If not previously installed, do so now.Then load the functions:
      library("pwr")

### The t Distribution
#The functions available in R are:
# dt(x, df, ncp, log = FALSE)
# pt(q, df, ncp, lower.tail = TRUE, log.p = FALSE)
# qt(p, df, ncp, lower.tail = TRUE, log.p = FALSE)
# rt(n, df, ncp)

# Parameters are the same as for binomial and normal functions
# with the addition of df (degrees of freedom) and ncp (noncentrality
# parameter). ncp will be used in power calculations.

## The dt Function
# Gives the density (essentially the height) of the t distribution at x.
# For the central t distribution, omit ncp.
# Also omit log = FALSE, unless log(p) is desired; in that case, log.p=TRUE.

#Here's a  comparison between the t and normal distributions.
      round(dt(seq(-2,2,1),6),3)		# There are 6 df
 	 round(dnorm(seq(-2,2,1)),3)
# Although both distributions are symmetric, the normal distribution
# has a higher peak and is lower at the tails

##Exercise 6.1
# Use R to obtain the t densities for 2, 20, and 200 df for the 5 x values
# in the preceding example. Compare these results with the normal densities 
# for the same x values

##The pt function
# This gives the probability of a value less than x unless "lower,tail=FALSE"
# is specified; in that case R reports the upper tail probability .
# If the lower tail is required, "lower.tail = TRUE" may be omitted.
 
#An example:
      pt(2,30,lower.tail=F) 		# P(t<2|df=30)

##Exercise 6.2
# Let x take the values 1.65, 1.75, 1.85, 1.95. Using the pt function,
# find the corresponding upper tail probabilities. Assuming alpha = .05,
# which cases are significant when the alternative is one-tailed? Two-tailed?
 
## The qt function
# This gives the quantile (x) that exceeds the lower tail proportion
# specified by p unless "lower,tail = FALSE" is stated; in that case,
# the reported quantile is exceeded by p. Consider the following examples:
# What is the score that exceeds 5% of a t distribution with 30 df?
      qt(.05, 30)
# What is the score exceeded by 5% of the t distribution with 30 df?
      qt(.05, 30, lower.tail=F)

## Exercise 6.3
# Assume a data set with 20 df.
#(a) Assuming H0: mu = 0 and H1: mu > 0, what are the cutoffs
# at the .01 and .05 levels? Use the qt function to answer this.
#(b) Now assume a two-tailed alternative. What are the cutoffs?
 
##The rt function
# This function allows us to draw random samples of size n from a t
# distribution with the specified df. For example,
      rt(20, 5)

##Exercise 6.4
# Let*s get a further sense of how the t distribution changes with df
#(a) Generate a sample of 2000 scores from the t distribution with 5 df and
# another from a t distribution with 60 df.
#(b)Using R*s qqnorm function, plot the Q-Q curve for each.
#(They can be plotted side by side in a two-panel box; see Session 2.)
#(b) Use the quantile function to find the .1 and .9 quantiles
# for each sample.
#(c) What do the results of (a) and (b) tell you?

 
### t Tests and Confidence Intervals
# We first need to read the data into the R console
# The file is "TC_IG.csv". Our data frame is
      tc.ig = read.csv(file.choose(),header=T)
      head(tc.ig)
      attach(tc.ig)

# Now test the difference between Control and Diet group means
      t.test(TC_Level~Treatment,var.equal = TRUE)
# This test assumes homogeneity of variance. The default is the
# Welch t test which adjusts df for unequal variances:
      t.test(TC_Level~Treatment)

# The results of the two t tests are not very different indicating 
# the group variances are not very different. We look at the two variances:
      aggregate(TC_Level,list(Treatment),var)
# The ratio of variances is not large, particularly given that
# the group sizes (n's) are equal (Table 6.3 on p. 137 of RDASA3 presents
# Type 1 error rates for various combinations of sample sizes and variances.)

##Exercise 6.5
# Read in the "Royer RT_Speed.csv" data.
# Construct confidence intervals and perform t tests on RT and on speed
# as a function of gender for the fifth graders. Note the means and standard
# errors for each of the analyses. Is there evidence of heterogeneity
# of variance. Is there evidence of any other violation of assumptions
# underlying the application of the t distribution?


### Cohen's Effect Size (Cohen's d)
# Continuing with the independent group TC data, calculate effect size
      tc.sd=aggregate(TC_Level,list(Treatment),sd); tc.sd
      tc.mean=aggregate(TC_Level,list(Treatment),mean); tc.mean
      diff=tc.mean[1,2]-tc.mean[2,2]
      diff
      s_pool= sqrt((35*(tc.sd[1,2]^2)+35*(tc.sd[2,2]^2))/70)
      s_pool
      es=diff/s_pool; es 			# effect size (d) = .384


### Confidence Limits on delta (noncentrality parameter) and Cohen's d
## Smithson has a script with four functions that provide limits
# on delta. I have extended them to provide limits on d as well. The 
# ptnoncentral functions are in the Functions.R script at my R website.
# Smithson's website contains other programs and a readme file.
# His URL is "http://dl.dropbox.com/u/1857674/CIstuff/CI.html"
# To illustrate (assuming you have the functions stored in your workspace)
      lot(1.6283,70,.95,2)     # t value, df, confidence,2 indpendent groups
#The output is:)
#[1] " low_delta          low_d     hi_p"
#[1] -0.35579309 -0.08386124  0.97500121
# These are the lower bounds on delta (noncentrality parameter of the
#    t distribution) and Cohen's d. For the higher bounds,
           hit(1.6283,70,.95,2)
#The output is:)
# [1] " hi_delta          hi_d           lo_p"
# [1] 3.60086460 0.84873193 0.02500593

## Exercise 6.6
# Given tne following summary information for an experimental and a control 
# group, calculate the effect size (Cohen's d) and .95 confidence bounds
# on it:
#  mE=4; mC=3.2	(group means)
# nE=18;nC=18    	(group sizes)
# vC=2.25; vE=1.44	(group variances)


### Data Analyses in the Correlated-Scores Design
# Detach the independent-groups file and clean the workspace
     detach(tc.ig)
	rm(diff,es,tc.ig,s_pool,tc.mean,tc.sd)

# Now read the correlated-scores data, "TC_CS.csv"
      tc.cs = read.csv(file.choose(),header=T)
      head(tc.cs)
# NOTE: The rightmost two columns have the between-subjects data.
# We can ignore those columns.
 # Now the test is
      t.test(tc.cs$Before,tc.cs$After,paired=TRUE)
# OR
      t.test(tc.cs$Change)
# Note that the results of the two tests are the same
 
## Exercise 6.7
# (a)Calculate the effect size (Cohen's d) for the correlated scores data.
# (b) use the lot and hit functions to get the confidence interval on d.
	rm(tc.cs)


###  Power of the t Test
# The R library folder has a power.t.test function. However, the pwr
# package is more compatible with Cohen*s approach and that of G*Power,
# and provides calculations of n and power for several statistical methods.
# Assuming you have installed the package, load it:
      library("pwr")
# Assume Cohen's d = .47, desired power = .9, and test is one-tailed.
# Then, to find the sample size
      pwr.t.test(n=NULL,d=.47,power=.9, alt="greater") 
# For power = .9, we need 79 subjects in each group.
#For other possible parameter values, check help(?pwr.t.test).
 
 
### Trimmed Means and Trimmed t Tests
# A discussion of this topic appears on pp. 160-161 of RDASA3 . 
## The Trimmed mean
# Trimmed means are easily calculated with R. For example,
      set.seed(106033)
      x = round(rnorm(10,25,10)); x 	#n = 10,mu-25,sigma=10
      mean(x, trim=.1)                           
# mean after dropping x[1],x[10]  = 30.375
# We can check the result by dropping the lowest and highest
# scores from x, then taking the mean:
      x1 = sort(x);x1
      mean(x1[2:9])
# Clean up
	rm(tc.cs,x,x1)

## The Trimmed t Test 
# The trimmed.t function can be found in the Functions.R script
# t my R website. An example of its application follows.

# When analyzing our data in SPSS,we needed a trimmed (T) and winsorized (W)
# column. In order to permit comparison of the R and SPSS results, those 
# columns, together with the full untrimmed data, have been left in the
# file we will use to illustrate an R function to perform a test on the
# trimmed means. However, note that the R function requires only the 
# original untrimmed data. Also, the scores do not have to be sorted 
# in porder to use the R function.

# The data file presents memory data for drug and control groups. 
# We first read in the data set "Trimmed t Data.csv"
      mem.data=read.csv(file.choose(),header=T, sep=",")[,1:2]
      head(mem.data) 	
# We excluded W and T (winsorized and trimmed) scores used with SPSS, 
# but not needed here.

# Create two groups from the single colun of Y scores.
      C.group = subset(mem.data, Method=="Control", select="Y")
      D.group = subset(mem.data, Method=="Drug", select="Y")

# Applying the trimed t function to the C and D groups
# The parameters are the group names,n1 and n2,the trim from each tail,
# the number trimmed from each tail, and the quantiles for the trim
      trimmed.t(C.group$Y, D.group$Y, 25,25,.2,5,probs=c(.2,.8))

##Exercise 6.8
# Write a function in R to obtain the .95 confidence interval for the
# difference between the population means in the preceding example.
# Use the trimmed means and winsorized standard error in your formula
# for the confidence interval.
 
##Exercise 6.9
# Write a function for a one-sample trimmed t test. Use the rnorm function
# to generate 20 numbers. Check your results using any statistical software
# package or calculator you have.


# Clean your workspace 
	rm(C.group,D.group,mem.data)
# Use ls() or LS() to check. You should have only functions in the workspace
# and the RoyerRT and mult_time data frames which will be used in scripts
# for Part 2.


