#####                   SCRIPTS FOR PART 4 

## NOTE: In this part, we will use functions from the following packages:
# car, psych, polycor, mvtnorm, lmtest, nortest, MASS, moments, pwr,effects,
# MBESS, and gsl
# The "car", "moments", and "pwr" packages should have been installed in
# connection with previous parts. If not, install them now, together 
# with the others.

#####              		Script for Session 18
#####		Introduction to Correlation and Regression

### NOTE: A useful website, listing many functions for regression 
#  analysis and plotting is
#     http://cran.r-project.org/doc/contrib/Ricci-refcard-regression.pdf


### Tabling Descriptive Statistics
#  Read the "Royer Data.csv" file:
    Royer.data = read.csv(file.choose(),header=T,sep=",")
# We want the subtraction and multiplication data for the 28 3rd
# graders who have no missing scores for those variables. Therefore,
     submult=na.omit(subset(Royer.data,grade==3,select=1:6))
# NOTE: "select" chooses columns 1 - 6. "na.omit discards cases with
# missing scores.
    head(submult)
    info(submult)
     attach(submult)

### Descriptive Statistics
# We will construct a table of summary statistics similar to
# that provided -- for example -- by SPSS's Explore option.
 
# The minimum, lower hinge (@25th quartile), median, upper hinge,
# and maximum for subtraction and multiplication accurcy are
	qstats=round(cbind(fivenum(subacc),fivenum(multacc)),3)
# The accuracy means are
	meanacc=round(c(mean(subacc),mean(multacc)),3)
	sdacc =round(c(sd(subacc),sd(multacc)),3)

# The confidence interval bounds on the mean are:
     lower.bound=round(meanacc+qt(.025,df=27)*sdacc/sqrt(28),3)
     upper.bound=round(meanacc-qt(.025,df=27)*sdacc/sqrt(28),3)
	bounds = rbind(lower.bound,upper.bound)

## NOTE: t.test(subacc) and t.test(multacc) include the CI bounds in the result

# The trimmed accuracy means are
     trimmed.mean = c(round(mean(subacc,trim=.05),3),
          round(mean(multacc,trim=.05),3))

#  Kurtosis and skewness statistics
     library(moments)
     skewacc=c(round(skewness(subacc),3),round(skewness(multacc),3))
     kurtacc=c(round(kurtosis(subacc),3),round(kurtosis(multacc),3))
# See the script for Part 1 for further discussion and alternate formulas.

#  A summary table can now be constructed:
     table.sum=data.frame(rbind(qstats,meanacc,sdacc,trimmed.mean,bounds,
		skewacc,kurtacc))
	rownames(table.sum)=c("min","hinge.low","median","hinge.high","max",
		"mean","sd","trimed mean","low bound","high bound",
		"skewness","kurtosis")
	colnames(table.sum) = c("subacc","multacc")
	table.sum

##Exercise 18.1
# The trimmed means differ slightly from the SPSS trimmed means.
# Calculate the .05 and .95 quantiles for subacc and multacc using
#  "type = 6",then find the mean of the values between them.

# Now clean up the workspace
     rm(bounds,lower.bound,meanacc,sdacc,skewacc,kurtacc,table.sum, 
		trimmed.mean,upper.bound,qstats)


### Scatterplots 
## Using the "plot" Function
# Plot multiplication accuracy as a function of subtraction accuracy
#  One way to do this:
     par(fig=c(0,0.8,0,0.8), new=TRUE)  # set for new figure with
								# room for boxplot outside
     plot(multacc~subacc,xlab="Subtraction Accuracy",
        ylab = "Multiplication Accuracy",pch=19)

# To get a smooth curve that captures the trend in the data
     lines(lowess(multacc,subacc), lwd=2,col="red") 
# A least-squares straight line could also be added
     abline(reg=coef(lm(multacc~subacc)),lwd=2)

# Does the smoother have a definite curvature? If so, a linear model
#   may not be appropriate.

 #  Now to add boxplots    
    par(fig=c(0,0.8,.45,.9), new=TRUE)
    boxplot(multacc,horizontal=TRUE, axes=FALSE)
    par(fig=c(0.55,.92,0,0.8),new=TRUE)
    boxplot(subacc, axes=FALSE)    # Is there an indication of skew? Outliers?
#  "fig" sets the size, location of plots; play with the
#      values to see how they effect the result.
#  "New = True" tells R that the following object is to be
#      incorporated into the existing graph.

# Close the graph
      dev.off()


## Exercise 18.2
# Plot multiplication response time (multrt) as a function of multacc
# include boxplots on the axes.

     detach(submult)

## Using the "car" Package's "scatterplot" Function
# The approach here assumes the "car" package has been installed.
	library(car)
# The total cholesteeol ("tc") data will be ued extensively in this
#  and the following sessions. We begin by reading in the "TC_age_weight.csv" 
#  file  and plotting total cholesterol level as a function of age.
     tc.data=read.csv(file.choose(),header=T,sep=",")	
     head(tc.data)
     attach(tc.data)
# Note: weight is in kilograms

#There are several types of smoothers, and several options for
#  constructing them. We will use the scatterplot function for which we need
     scatterplot(tc~age,pch=19,lwd=2)
# pch dictates type of symbol, lwd dictates line widths
# The curved line is a loess smoother. a nonparametric function
# to capture the trend of the data. The dashed lines provide smoothed
# bounds on the smoother. The solid line is a linear regression line.
# These (and the boxplots) are defaults.

     dev.off()

Exercise 18.3
# Plot tc as a function of weight. Omit the linear regression line
# and spread but include an ellipse that contains the major mass of data. 
# Type "help(scatterplot)" to see how to do this.


### Regression and corrrelation coefficients
# Let's detach tc.data and re-attach the submult data frame.
     detach(tc.data)
     attach(submult)

# This time we plot multiplication RT as a function of multiplication
#  accuracy. 
      plot(multrt~multacc,xlab="Multiplication Accuracy",
        ylab = "Multiplication Response Time",pch=19)
     abline(reg=coef(lm(multrt~multacc)),lwd=2)

# The slope is  clearly negative. If we want the values of the 
#  intercept and slope, one way is
     mult.model=lm(multrt~multacc); mult.model
# The intercept and slope of the least-squares line are
# Intercept      Slope  
#  10.35458     -0.06734 

	dev.off()

# For considerably more information about the plot,
     summary(mult.model)
#  The residuals are the differences between observed and predicted
# Y ("Y_hat",predicted by least-squares line)
# Note that the Pearson-product correlation coefficient is the (negative) 
# square root of the reported squared correlation.
# The summary tells us that both intercept (10.355) and slope (-.067)
#  differ significantly from zero.

#  If all we want is a test of whether the slope differs from zero,
     anova(mult.model)		#F = 8.10, p = .0085

## Exercise 18.4
# Is there a significant relation between subtraction and multiplication 
# response times? Plot the data, then perform the analysis to answer the
# question.

## Summary Statistics
    acc.model=lm(multacc~subacc)
    summary(acc.model)
# The summary provides information about the distribution of the
# residuals, and estimates of regression coefficients, and their standard   
# errors, and significance tests.
# The F-statistic is the square of the t for subacc (slope). 

# We may also want confidence intervals on the parameters
     confint(acc.model)
#  .95 is the default; else, -- for example 
     confint(acc.model,level=.90)
# Both the summary p values and the CI indicate that the intercept
# does not differ significantly from zero, and that the slope does.


### Influence Plots
# Make sure the car package is attached by typing search()
	influencePlot(acc.model,id.method="identify")					ify")
# id.method = "identify" means that clicking on a point will
#  reveal the case number. Useful in identifying possible outliers.
  					
## Some Definiitions
# Ordinate values: 
#	the ith studentized residual = y(i) - y_hat(i) 
#	divided by the standard error with the ith score deleted.
# Abscissa values: 
#	The ith Hat value (bivariate case) is
#	h(i)=(1/N)+[x(i)^2]/SSX where x(i)=X(i)-X_bar, 
#	and SSX is sum[(X-X_bar)^2]. The hat value is a measure of
# 	leverage, the effect X(i) has on the regression coefficients

# Click on the larger circles in the plot.Cases 5, 25, and 2 are the
# largest circles, and are potential outliers.
# Now close the plot. First right-click on it, and select "stop." 
# "Stop" causes the StudRes, Hat,and CookD values of the selected points
# to appear on thw R console screen. 

# CookD is Cook's Distance and is reflected in circle size. It is
# a measure thst incorporates both the size of the residual and the
# leverage of a data point (roughly, how much in error is the prediction 
# and how much of a role does the X value play?). 

       dev.off(2)

## Deleting an influential point
# What happens if we delete case 5?
     acc.model2=lm(multacc[-5]~subacc[-5])
     summary(acc.model2)

# Comparisons with the original output based on all 28 cases:
#              intercept slope r-squared
#   acc.model   1.633	  .833    .353
#   acc.model2 23.493      .642    .153

# The parameters are greatly changed and the correlation reduced
# by the elimination of a single outlier.


## Plotting Groups of Data Points
# We might wish to plot separately the data for the boys and girls. One way
# is to have two plots in a single frame, one for each group (see Session 2)
# But if we want to compare groups, plotting the two sets of points
# in one plot is better:
      scatterplot(multrt~subrt|gender, smooth=F,by.group=T)
# We conditionalized on gender, eliminated the smoother line to reduce
#  clutter, and specified that we wanted separate least-squares lines
#  for the two groups. We can see that the lines are similar for boys and girls.

	dev.off()

# We might wish to examine the statistics for each group.
     boys=subset(submult,gender=="Male")  
     girls=subset(submult,gender=="Female")

   	boys.lm=lm(multrt~subrt,data=boys)
     girls.lm=lm(multrt~subrt,data=girls)

	summary(boys.lm)
	summary(girls.lm)
# The statistics are quite similar

# The summary function gives the stats for the two models. Alternatively, we
# can just compare the coefficients and the correlation coefficients:
      c=rbind(coef(boys.lm), coef(girls.lm))
      R =rbind(cor(boys$multrt,boys$subrt), cor(girls$multrt,girls$subrt))
# "cor(X,Y)" yields the X-Y correlation; see "?cor" for more details
# "coef(model)" yields intercept nd slope
      reg.stats=cbind(c,R)
# Labels would be nice
     colnames(reg.stats)=c("intercept","slope", "Pearson r")
     rownames(reg.stats)= c("Boys","Girls")
     reg.stats

## Exercise 18.5
# Add 10 to all multrt and subrt. (a)What effects, if any, does this have
# on reg.stats? (b) What effect does this have, on the  statistics
# from the ungrouped data?

## Exercise 18.6
#  Write a function to create reg.stats for any number of groups.

# Clean up
	detach(submult)
	rm(acc.model,acc.model2,boys,boys.lm,c,girls,girls.lm,mult.model,
	R,reg.stats,tc.data)
# Note that we have kept the Royer.data and submult in our workspace. They
# will be used in future sessions.

############################################################

#####              Script for Session 19
#####			 More about Correlation

### Inferences about Rho
## Testing H0: rho = 0
# We test the correlation coefficient between subtraction and multiplication
#  response time:
     attach(submult)
    cor.test(subrt, multrt,method="pearson")	
# t = 2.23, df = 26, p-value = 0.035; 95 percent confidence interval:
# 0.03225882 0.67302672 (based on Z transform)
# See the "fisherz" function in the "psych" package). 

#NOTE: The estimate of rho (.401)rests on the assumptions 0f s linear relation
# between X and Y, and bivariate normality. Simulation studies indicate
# that violations of the normality assumption have little effect 
# on Type 1 errors although power may be reduced. qqplots 
# (the qqnorm function) will aid in assessing departures from normality.
# We will consider methods for examining the validitry of assumptions 
# in Session 20.
     
## Testing Values of rho Other than Zero
#  Suppose research suggest rho(X,Y)=.6. Our analysis of Royer's data
#  with n = 28 yieldes r = .401. Can we reject H0: rho = .6?
     n=28; r=cor(subrt,multrt); rho=.6
     se_Z=1/sqrt(n-3)
     Z_r = (1/2)*log((1+r)/(1-r))
     Z_rho = (1/2)*log((1+rho)/(1-rho))
     z = (Z_rho - Z_r)/se_Z
     z; 2*pnorm(z,lower.tail=F)    # two-tailed test
# z = 1.344; p = .179. We can not reject H0. The .95 confidence interval is
     UB = Z_r+(1.96*se_Z);  LB = Z_r-(1.96*se_Z)
     UB; LB   # The .95 interval on Z_rho is .032 to .816. 
# We can convert the bounds on Z_rho to bounds on rho. For the upper bound,
     E_UB = exp(2*UB)
     UB_rho = (E_UB-1)/(E_UB+1); UB_rho
# The upper bound on rho is .673. Simlarly, the lower bound is 
      E_LB = exp(2*LB)
     LB_rho = (E_LB-1)/(E_LB+1); LB_rho    
# lower bound on rho = .032
# These are the same bounds given by the cor.test function

##Exercise 19.1
# Calculate the correlarion for multacc and subacc. Test for significance
#   using the the t test on r and the z test on the transformed r. Then find
#   the confidence interval for R.

## Exercise 19.2
#  Write a function to test rho = RHO where RHO = any value between -1 and +1. 
# The function should include confidence bounds on RHO. Test your function
# by comparing results with those obtained for the submult RT's and with
# your answer to Exercise 19.1.

# The preceding tests and confidence intervals (as well as other useful
#  functions) are also available in the "psych" library. If not
#  already installed, install.packages("psych"). Then,
     library(psych)


### The "psych" Package's  r.test Function
## Testing H0: rho = 0
   n = 28; r= cor(subrt,multrt)
   r.test(n,r)
# r.test from the "psych" package provides the same results as cor.test did

## Testing the Difference Between 2 Independent Correlations.
# Using Fisher's Z transform, this is a straightforward extension of
# the preceding developments. The variance of a difference is the sum of
#  the variances if they are based on independent observations. Therefore,
#     se_diff.Z = sqrt(1/(n1-3)+1/(n2-3))
# where n1 and n2 are the group sizes. The standard normal z test is
#  z = (Z1-Z2)/se_diff.Z. We apply this to compare accuracy correlations for
# Royer's 3rd and 5th graders. We then use the r.test function to do
#  the same test. First construct the 5th grade data frame.
    submult5=na.omit(subset(Royer.data,grade==5,select=1:6))
     n3=nrow(submult)		# How many rows?
     n5=nrow(submult5)
     r12=cor(submult$subacc,submult$multacc)
     r34=cor(submult5$subacc,submult5$multacc)
# fisherz from the "psych" package (is it installed and loaded?)
     Z12 = fisherz(r12)
     Z34 = fisherz(r34)
#Then,
     se_diff.Z = sqrt(1/(n3-3)+1/(n5-3))
     z = (Z12-Z34)/se_diff.Z
     z;   2*pnorm(z,lower.tail=F)
# z = 1.517257 and the 2-tailed p =  0.1292018 

# The r.test function in the "psych" package is simpler
     r.test(n=n3,r12,n2=n5,r34)  			# The result is
# "z value 1.52    with probability  0.13"	Same as previously  

## Exercise 19.3
# Calculate the correlations of the winter and spring Beck depression
#  scores (beck_d1, beck_d2) separately for men (sex=0) and women (sex=1).
#  Use the Beck_D data file that can be downloaded from my website.
#  Test whether the correlations differ significantly.

##  Testing hypotheses about Dependent Correlations
# We have the following correlation matrix from the Seasons data on
# my R website
      cor.mat = cbind(c(1,-.129,.513,.070),c(-.129,1,-.198,-.093),
      c(.513,-.198,1,.152),c(.070,-.093,.152,1))
     colnames(cor.mat)=c("Age","Height","TC","BMI")
     rownames(cor.mat)=c("Age","Height","TC","BMI")
     cor.mat
# To protect against Type 1 errors,Steiger proposed a preliminary test of
#  whether any correlations differed significantly from zero First,
#  convert the six off-diagonal entries to Fisher Z's:
 3    Z.cor=fisherz(c(-0.129,0.51,0.07,-.198,-.093,.152))
#  (N-3)*sum(Z*Z) is distributed as chi square on 6 df's(number of Z's). 
#   The correlations were based on data from 207 individuals. Therefore,
     chi_square = 204*sum(Z.cor*Z.cor)
     pchisq(chi_square,6,lower.tail=F)
# The p value is virtually zero; clearly, one or more of the correlations
# differs significantly from zero. We now can test whether the correlation
# between TC and Age differs significantly from that for TC and BMI.
     r12 = .513; r13 = .152; n = 207
     r.test(n,r12,r13)
# The result is " z value 4.18   with probability  0"

# Clean up the workspace but save the submult data frame and cor.mat
# for future use.
	detach(submult)
	rm(chi_square,E_LB,E_UB,LB,LB_rho,n,n3,n5,r,r12,r13,r34,rho,
		Royer.data,se_diff.Z, se_Z,submult5,UB,UB_rho,z,Z.cor,Z_r,Z_rho,
		Z12,Z34)
	

### Partial Correlations: Calculations
# Suppose we want to correlate Age and TC, adjusting for the contribution
#  of BMI to each. In essence, we wish to correlate two sets of residuals,
#  that when age is predicted by BMI and that where TC is predicted by BMI.
# NOTE: r1=r(Age, TC), r2=r(Age, BMI), r3=r(TC,BMI)
     r1 =.513; r2 = .07; r3 = .152  
# The partial correlation of age and TC adjusted for BMIis given by
     r_AxTC_BMI = (r1-r2*r3)/sqrt((1-r2^2)*(1-r3^2)); r_AxTC_BMI  #r=.510
# Easier to use the "partial.R" function in the "psych" package; note that
# variables 1, 2, 3, 4 = age, height, TC, BMI
      partial.r(cor.mat,c(1,3),4)  # Partial out column 4 (BMI)
# We can even adjust for more variables
           partial.r(cor.mat,c(1,3),c(2,4))  #Partial out Height and BMI
# We also can adjust several correlations for the effects of 1 or more others; e.g.
      partial.r(cor.mat,c(1,2,3),4)
# which adjusts the correlations among the first three variables for the 4th


### Partial Correlations: Tests and Confidence Intervals
# The "Functions,R" script contains a function for doing the
# tests and provides a confidence interval on the partial correlation.
# The form of the function is "function(n,k,rho,r,alfa)"
# where n = no. of cses, k = no. of variables factored out,
# r is the correlation, and alfa is the significance level
# (confidence = 1-alfa)
#  E.g. if H0: partial Rho = 0, for cor.mat, the command might be
         partial.r.test(207,1,0,.512,.05)
# and the output is 
#     [1] " Statistic  p-value   Upper Bound   Lower Pound"
#     [1] 16.62759  0.00000  0.70300  0.42787

# If we hypothesize partial rho = .5, the command is
      partial.r.test(207,1,.5,.512,.05)
# and the output is
#     [1] " Statistic  p-value   Upper Bound   Lower Pound"
#     [1] 0.22982 0.81823 0.70300 0.42787

# The parameters are: n,number of variables partialed out, rho,r,alpha).
# In the output, "Statistic" is either t (when rho = 0) or z. The CI
# bounds on partial rho are based on the normal distribution.

## Exercise 19.4
# Attach submult and get the correlations:
#      submult2=cbind(subacc,subrt,multacc,multrt) 
#      royer.cor=cor(submult2)
#  Now find the correlation between multrt and multacc after partialing out
#  the subrt and subacc variables (a) using the "partial.r" function of the
#  psych package; (b) by finding the residuals for mult rt and multacc
#  when the subtraction  ariables are the predictors; i.e..
#    res1 = resid(lm(multacc~subacc+subrt))
#    res2 = resid(lm(multrt~subacc+subrt))
#  The correlation of these residuals should be the same as the result in (a).

## Exercise 19.5
# using the partial.r.test function, test the r from Exercise 19.4
# against rho = 0, and get the confidence interval.


### Semi-Partial Correlations
# To correlate Age with TC when the contribution of BMI to TC is removed,
#  we can use the general formula
#       (r1-r2*r3)/sqrt(1-r3^2)
#  where r1 = cor(Age,TC); r2 = cor(Age,BMI); r3=cor(TC,BMI)
# Note that this is partial.r(cor.mat,c(2,3),4) * sqrt(1-r2^2)
# The same result can be obtained by correlating Age with resid(lm(TC~BMI))

## Exercise 19.6
# Use the two methods just outlined to find the semi-partial correlation of 
#  Age and TC when BMI is partialed from TC


### The Point-Biserial  and Phi Correlations
## The point-biserial correlation is applicable when one variable is
#  continuous and the other is dichotomous. It is identical to the Pearson
#  r and therfore either cor.test or r.test can be applied to the data.

##   The Phi Coefficient is applicable when two variables are dichotomous.
#  The"psych" package contains a phi function. We might have
	freq=matrix(c(14,6,12,8),nrow=2)
	colnames(freq)=c("Boys","Girls")
	rownames(freq)=c("Pass","Fail")
     phi.freq = phi(freq); phi.freq   # = .1
#  To test this correlation
    N=40
    pchisq(N*phi.freq^2,1, lower.tail=F)
# p = .527; we conclude that p(Pass) does not depend on sex.

		
## Exercise 19.7
# (a) Using the submult data frame, find the point-biserial r for gender
#  and multacc, then test against H0: point-rho = 0.
# (b) Now asssume multacc > 70 = Pass and multacc <= 70 = Fail. Find the
# phi voefficient for gender and pass_fail. Test this coefficient.


### The Biserial and Tetrachoric Correlations
# These are available in the "psych" and "polycor" packages.
     
## Biserial Correlation
#   The data set for the biserial correlation, like that for point-
#  biserial, has one dichotimized factor and a continuous. The difference
# is that it is assumed that the dichotomy represents an underlying normally 
#  distributed variable.

#  To geneerate a bivariate normal data set: 
	install.packages("mvtnorm")	# Unless installed ewarlier
	library(mvtnorm)
#  Assume the following data set
     set.seed(105324)
     xy0=data.frame(round(rmvnorm(50, c(0,6),
           matrix(c(1, 0.60, .6, 2), 2,2))))  # bivariate normal sample
	head(xy0)
# Now dichotomize the X1 variable
     X=1*(xy0$X1>0)
     X.data=data.frame(cbind(X,10000*xy0$X2))
     colnames(X.data)=c("Response","Income")
     head(X.data)
# The point-biserial coefficient would be
    cor.test(X.data$Response,X.data$Income)  # r is .505

# To get the biserieal coefficient:
	library(psych)			# If not already loaded (to check "search()")
#  If we assume Response represents a normal continuum,
     biserial(X.data$Income,X.data$Response)  # From the "psych" package,
#   cor=0.659. Compare with the point-biiserial result

#  NOTE: if X has more than two ordered categories, ployserial(Y,X) can
#   be used (from the "psych" or "polycor" package)

## Tetrachoric Correlation
# This requires the psych library to be loaded.

# Using the following table, we assume that the column factor is pass/fail,
#  that the rows are two age groups. and that underlying each are 
#  continuous variables with a bivariate normal distribution. The frequencies 
# are the same we used in illustrating the phi function  (phi = -.24).
   Age1 = c(40,20)
   Age2 = c(35,5)
    X = rbind(Age1,Age2)
      colnames(X) = c("Pass","Fail"); X
# And the coefficient is now
     tetrachoric(X)    # = -.42
# NOTE: psych and polycor packages also provide a polychoric function.


### Correlations Based on Ranks
# We use the response time data (subrt and multrt) from the Royer data.
# Recall that for the original RT, r=.401, and the .95 confidence
#  interval was: LB = .032, UB = .673; p = .035. 

## The Spearman Correlation
# The p value (H0: rho=0) is given by
     cor.test(submult$subrt,submult$multrt, method="spearman")
# In this case, Spearman's rho = .522, p = .004

## Kendall's tau
# The p value (H0: tau=0) is given by
     cor.test(submult$subrt,submult$multrt, method="kendall")
# tau -.352, p = .009

# Clean up
	rm(Age1,Age2,cor.mat,freq,N,phi.freq,r_AxTC_BMI,r1,r2,r3,
		X,X.data,xy0)

################################################################

#####              Script for Session 20
#####          More about Bivariate Regression

### Regression Analysis of Data from a One-Factor Experiment
# A simple experimental data set 
     size = rep(2*(1:4),each=5)    # set size
     rt = round(c(418,428,410,445,471,475,455,418,524,516,537,500,480,511,
             529,550,617,590,608,548))
     summary(lm(rt~size))    #intercept and slope, tests,SE's, R-squared
     confint(lm(rt~size))    # Adds .95 confidence intervval on b0, b1
     anova(lm(rt~size))      # Adds sums of squares

## Exercise 20.1
#  SPSS's Beta (standardized coefficients) can be obtained by standardizing 
#  both variables, then obtaining the coefficients. Do this and verify that
#  Beta = .873.

# clean up the workspace
	rm(size, rt)
    
### Testing the Difference between Two Independent Slopes 
# Read the "Seasons.csv" file 
     seasons = read.csv(file.choose(),header=T,sep=",")
     tc.m=na.omit(subset(seasons,sex==0,select=c(age,tc))) 
# The subset of males; na.omit => drop cases with missing scores
	tc.w=na.omit(subset(seasons,sex==1,select=c(age,tc)))	# Data of Women

## The tc x age Slopes for Each Sex
     model.m = lm(tc.m$tc~tc.m$age)
     model.w = lm(tc.w$tc~tc.w$age)
     b1.m= summary(model.m)$coefficients[2,1]	# slope for men
     b1.w= summary(model.w)$coefficients[2,1]	# slope for women
     round(c(b1.m,b1.w),3)	            # .198, 1.712

## The Standard Errors of the Two Slopes 
     df.m = length(tc.m[,1])-2    			# = 218
     df.w = length(tc.w[,1])-2    			# = 209
     ss.m = (df.m+1)*var(tc.m$age) 		 # SS(Age|Men)  = 30944.98   
     ss.w = (df.w+1)*var(tc.w$age)   		 # SS(Age|Women) = 28714.56
    sse.m = sum(residuals(model.m)^2) 		# SS(error|Men)= 319539.8
    sse.w = sum(residuals(model.w)^2) 		#SS(error|Women)= 244722.5
    se=sqrt((sse.m+sse.w)/(df.m+df.w)); se 	# = 36.35185
 
##  The t Test
     se.diff = se*sqrt(1/ss.m + 1/ss.w)
     t = (b1.w-b1.m)/se.diff; t    # = 5.079
     
## The  "b.diff.test" Function 
# This is in the "Functions.R" script. It contains a function to do the
# preceding analysis. It returns the slopes, se of the difference,
# t and p values. The general form of the function is
#     b.diff.test=function(data.1,data.2,x.1,x.2,y.1,y.2) 
# 1 and 2 refer to two independent data frames with no missing scores
# To use the function for the current data set:
     b.diff.test(tc.m,tc.w,tc.m$age, tc.w$age,tc.m$tc,tc.w$tc) 


### Least-Squares Linear Predictions
## Fitting the Obtained Scores
# Although we can substitute the X value into the linear equation once
# we have values for the intercept and slope, R provides a way to
# obtain predicted (Y_hat) values. For the tc scores for men
     y_hat=predict.lm(model.m,se.fit=T,interval="confidence")
	head(y_hat$fit)	# First six cases with predicted scores and CI limits
	head(y_hat$se.fit)	# std. errors for the 1st 6 cases
	c(y_hat$df,y_hat$residual.scale)	#df, std. error of residuals
# If the interval parameter is omitted, the default is "none"
#  "fitted(model.m)" also gives just the predicted values

# To predict a specific case with its stamdard error; e.g., case 5, 
      round(c(y_hat$fit[5,],y_hat$se.fit[5]),3)		# the result is
#	    fit     lwr     upr         
# 	   221.632 216.514 226.751   2.597
# The "fit' is the predicted score for case 5, "lwr" and :upr" are 
# confidence bounds on that prediction, and the 4th number is the SE
# of the prediction.

# Let's look at the formulas underlying these four values. For example,
# Compare this predicted value for case 5 with that for the prediction
# equation, b0+ b1*x:
     b0.m = coef(model.m)[1]
     b1.m = coef(model.m)[2]
     round(b0.m+b1.m*tc.m$age[5],3)
# The formula for se.fit is obtained as follows:
     se.m = sqrt(sse.m/df.m); se.m	
# s(y.x|men), the std. error of the residuals
     h.m = 1/(df.m+2) + ((tc.m$age[5]-mean(tc.m$age))^2)/ss.m; h.m
# h(i) reflects the leverage of the ith X value; its effect on the
# regression coefficients increases as its squared distance from the
# mean increases relative to the sum of squares.Then, 
	se.fit = se.m*sqrt(h.m); se.fit
# This is identical to y_hat$se.fit[5]

# Predicting the Score for a New Case 
# Assume age = 49
      attach(tc.m)     
      model.m = lm(tc~age)        # define the  model
      newdata=data.frame(age=49)
      y_hat2=predict(model.m,newdata,interval="prediction")
      rbind(y_hat$fit[5,],y_hat2)
# Althougth the predicted tc is the same, the prediction interval 
#  is much wider. The output is:

#        fit      lwr      upr
#  221.6324 216.5142 226.7507    # confidence interval
#1 221.6324 146.0020 297.2629    #  prediction interval

#  The formula for the prediction SE for a new case is
    se.m_new=se.m*sqrt(1+h.m) # =38.37346; note the "1" aded to leverage
    t_crit=qt(.025,df.m,lower.tail=F); t_crit
    LB = 221.6324-t_crit*38.37346
    UB = 221.6324+t_crit*38.37346
    c(LB,UB)
#   [1] 146.0019 297.2629    # the answer when interval = "prediction"

## Exercise 20.2
# Find confidence and prediction intervals for the predicted tc of a
#  woman of age 49.

# Clean up. Do not remove tc.m or tc.w, or the seasons data frame.
# We will return to these data later. For now, the Royer arithmetic data
# (the "submult" data frame) provides a somewhat better example for 
# diagnostic plots.
	detach(tc.m)
	rm(df.m,df.w,b0.m,b1.m,b1.w, h.m,LB,model.m,model.w,newdata,
	   se,se.diff,se.m, se.m_new,ss.m,ss.w,sse.m,sse.w,
	   t,t_crit,UB,y_hat,y_hat2)


### Diagnostics Plots for Regression Analyses
## Plotting the Data
# If you have not saved the data frame "submult" in your workspace,
# return to the start of Session 18 and run the commands to recreate it.
#  Now  attach submult and recreate the model
     attach(submult)
     mult.model=lm(multrt~multacc)
	mult.model				# Gives the intercept and slope

## Plot(mult.model) 
# This makes available six graphs that are helpful
# in identifying outlying residuals and outlying X values. Only 
# four of these are presented by default, but the other two
# can be accessed. To see the four default plots

   plot(mult.model,qqline = T)	#Hit "Enter" to see first figure	
# see "help(model.lm)" for further information

# NOTE: if we precede the plot command with "op = par(mfrow=c(2,2))", 
#  we see a single figure with 4 panels. Omitting this "op" command, 
#  we see each figure in turn; it is easier to examine these larger figures.
 
#  Consider the first figure. It plots reresiduals (tc-predicted tc)
#  against y_hat [fitted(model.w)]. Note the very large residuals
# (absolute values) for cases 5, 25, and 26. These are the points
#  labeled in the figure and may be outliers. The variation in
#  the residuals suggests the model may be misspecified. Nonlinearity and 
# heteroscedasticity are possibilities but this may be a function of 
# the presence of the three labeled cases.

# In the Q-Q plot (next figure), these cases are again noted.
# There is evidence of nonnormality in the curvature of the line formed
# bny the points.
 
#  The third figure plots sqrt(abs(standardized residuals)) vs y_hat.
#  The variation in ordinate values remains after standardization.
#  NOTE: The ordinate is labeled differently by dfferent statistical
#  packages. RDASA3 refers to the core term on the ordinate as the
#  internal studentized residual; see p.518 for the formula. 

# The fourth figure plots the standardized residuals against their
# leverage values. Leverage is a function of the squared distance of X
#  from mean(x) and reflects the influence of a score on the
#  estimated regression function. Leverage (the "hat" value) is
#  (1/N) + ((x_i-mean(x))^2)/sum(x-mean(x)); e.g. for the 5th case,
      N = 28
      d=multacc-mean(multacc)
      h_5=1/28 + (d[5]*d[5])/sum(d*d);h_5	 #h_5 = 0.3069863
# To get all hat values, 
      h = hatvalues(mult.model); h
# and the 5th value is simply
	hatvalues(mult.model)[5]

#  One proposed criterion for outlying predictors is h > 2(p+1)/N
#  where p is the number of predictors; 1 in the bivariate case. Therefore,
     h[h>4/N]
#  Only cases 5 and 25 are outlying predictors by this criterion
 
#  The dashed lines in the fourth plot represent two criteria
#  for Cooks' distance, a measure that incorporates both leverage and the
#  size of the residual; it measures the influence  of each data point on
# all the fitted values. Values within the ellipse formed by the dashed red 
#  lines are plausible. By this criterion, only case 5 is a clear outlier.
#   However, have R calculate all the values of Cook's d:
      sort(round(cooks.distance(mult.model),3),decreasing=T)
#  Case 5 has a very large Cook's value but cases 25 and 26 
#  also have large values of d relative to the others. A clearer sense 
#  can be obtained by
     plot(mult.model,which=4) # Select the 4th of 6 plots; the default
                              #  was plots 1 - 3, and 5, already viewed.
    dev.off()

# One criterion proposed for CD is F(.5,p+1,n-p-1). In this example
     qf(.5,2,26)  	# =.711; by this criterion, only case 5 is an outlier;

## The Influence Plot
#  Another way to identify outliers is the influence plot of the 
#  studentized residual as a function of its "hat" (leverage) value.
# The ith studentized residual (referred to as externally studentized in
# some sources) is the residual [Y - mean(Y)] divided by a standard error
# calculated with the ith case deleted. The size of the circle reflects
# the influence of the case as measured by Cook's distance. The function is
# available in the "car" library:

	library(car)
     influencePlot(mult.model,id.method="identify")

#  "identify" allows us to click on any circle to obtain the case number.
$ Again, we see the influence of cases 5, 25, and 26 when we click on
#  the three largest circles.

# To turn off the active plot, right-click on it; select "stop"; then
	dev.off()

## Exercise 20.3
# Delete the three possible oujtliers by
#     submult2=submult[-c(5,25,26),]  # Note how we create a new data frame
	                                 # by removing specified cases
# Do a complete regression analysis for multiplication rt as a function
# of multiplication accuracy for both data frames. How has removal of the 
# possible outliers affected the correlatio n, regression coefficients,
#  and the standard error. What changes do you note?


### Assessing the Effects of Deleting Data Points 
# R provides a general approach to assessing the effects of deleting
# data points. 
		
## The Standardized Change in Regression Coefficients
# When a point is deleted the change in regression coefficients for
# the multiplication response times is
     change=data.frame(round(dfbetas(mult.model),5))
	head(change)

# The slope and intercept exhibit the largest absolute change when 
# case 5 is deleted, followed by cases 25 and 26. We can see this by
# ordering the absolute change in intercepts:
	order(abs(change[,1]),decreasing=T)
# and if we order the absolute change in slopes:
	order(abs(change[,2]),decreasing=T)

# One criterion for change is 2/sqrt(N),  0.3779645 in this example;
# Cases 5, 25, and 26 are the only ones with absolute changes in either 
# the intercept or slope  exceeding this criterion. We can see this by
# sorting "change" with respect to the  intercept:
	head(sort.data.frame(abs(change),1,decreasing=T))

## The Standardized Change in Predicted Values 
# When a point is deletedthe change in predicted value is
     d.fit=dffits(mult.model); 
	sort(d.fit)
# Again, cases 5, 25, and 26 stand out. A criterion for the significance
# of the change in predicted value is given by 
     p=1					# p = no. of predictors
    d.fit[abs(d.fit)>2*sqrt((p+1)/N)]
#  adding to our sense that these 3 cases, especially 5, are outliers.

#Clean up
	detach(submult)
	rm(change,d,d.fit,h,h_5,mult.model ,N,p) 
# Note that we have kept several data frames in the work space for 
# further analyses. Use the LS() function to see which ones. 
 

### Tests of Assumptions
# We return to the tc data, specifically those for women. Before 
# considering statistical tests, let us look at the data.
	attach(tc.w)
      model.w=lm(tc~age)
	summary(model.w)
# From the output,
     sy.x =34.22		# The residual standard error
# Now get the standardized resduals for plotting purposes
	res=residuals(model.w)
     std.res = res/sy.x

#  NOTE: This is the SPSS definition of standardized residuals;
# As we discussed when #  considering the plots of the Royer data
# at the beginning of this #  session, R's definition of standardized
@ scores is different.

# The "car" library must be loaded to access the scatterplot function
## Plotting Resuiduals
     y_hat = fitted(model.w)  #Less output than "predict" function
     std.y_hat = (y_hat-mean(y_hat))/sy.x
     scatterplot(std.res~std.y_hat,pch=19,lwd=2, boxplot=F,
     ylab="Standardized Residuals", xlab = "Standardized Predicted Y")

#  There is no obvious departure from linearity (compare the
# smoother with the best fitting striaght line), nor obvious
#  heteroscedasity. However, we will provide statistical tests.

# Close the plot
     dev.off()

## An F Test of the Linearity Assumption
# Although the residual plot does not clearly depart from linearity,
#  we may wish to test the hypothesis that the true function is linear;
# i.e., we wish  to test whether the points depart significantly from a
# straight line. One approach is provided here.

# (1) Get SS_res (reiduals) and df_res :
	SS_res=anova(model.w)[2,2]; SS_res		# = 244722.5
	df_res = anova(model.w)[2,1]; df_res	# = 209
# Note: We pick off the entries in the 1st 2 columns of the 2nd row 
# of the ANOVA output

# (2) Get SS_pure_error (within age levels variability)
	model.w2=lm(tc~factor(age))
# Note: if age is not treated as a factor, the residuals are from
# the best-fit line; i.e., 244723 again
	SS_pure_error=anova(model.w2)[2,2];SS_pure_error		# = 200765.7
	df_pure_error =  anova(model.w2)[2,1]; df_pure_error	# = 164
# (3) Get SS_nonlinearity
	SS_nonlin=SS_res-SS_pure_error; SS_nonlin   			# =43957
	df_nonlin=df_res - df_pure_error; df_nonlin			# = 45
	
# (4) Do the F test of nonlinearity; F = MS_nonlin/MS_pure_error
     F =(SS_nonlin/df_nonlin)/(SS_pure_error/df_pure_error); F       
#F =.80;  the p-value is
     pf(F,45,164, lower.tail=F)       # = .811
# The departure from linearity is not significant.
			
## Testing for Quadratic Curvature
#  The smoothed function (curved line) in our plot of the residuals appeared
#  to have a bump, suggesting a quadratic form. Although the departure
#  from linearity was not significant, we might wish to see if adding
#  a quadratic term would improve our fit significantly.
#  We center our variables to minimize multicolinearity (see Chapter 21
# or any discussion of multiple regression for a definition). Therefore,
       tc.c=tc-mean(tc)
      age.c=age-mean(age)
	 age.c2=age.c*age.c
	 model.wc = lm(tc.c~age.c)		# linear model restated
      model.wc2 = lm(tc.c~age.c+age.c2)    # quadratic model
#  Now to test the models against each other:
      anova(model.wc, model.wc2)
#  Although the RSS (residual sum of squares) is smaller for model 2,
#  the F test reveals no significant difference. Another way:
     summary(model.wc2)
#  The quadratic coefficient is not significant.
# We can also compare the AIC (Akaike information criterion) for the
# two models:
      AIC(model.wc)        #2093.613
      AIC(model.wc2) 	   #2095.303
#  The linear model provides a (very slightly) better fit.

## The Harvey-Collier Test of Linearity
# The "lmtest" package provides another way to test for nonlinearity.
# If the package is not installed, do so. Then,
     library(lmtest)
     harvtest(model.w)
#  The result is
# HC = 0.6761, df = 208, p-value = 0.4997; HC is the Harvey-Collier 
# statistic. As we surmised from our plot of standardized residuals, 
# and consistent with the previous tests of linearity,
# the departure from linearity again is not significant.

## Testing the Homoscedasticity Assumption
# The "car" package provides the following:
     ncvTest(model.w)		# p =.48
# We can not reject the null hypothesis of constant variance
    
## Testing Departures from Normality
#  If not done yet, install the "nortest" package. Then,
     library(nortest)
# There are 5 different tests of normality in the package. We will
# use the Anderson-Darling test.
     ad.test(tc)
# The result is
#     A = 0.2839, p-value = 0.6284
# there is no evidence of a departure from normality in the population

# Clean up
	detach(tc.w)
	rm(age.c,age.c2,df_nonlin, df_pure_error,df_res,F, model.w,model.w2,
	    model2.aov,model.wc,model.wc2,res,SS_pure_error,SS_nonlin,SS_res,
	   std.res,std.y_hat,sy.x,tc.c,tc.m,tc.w,y_hat,se.fit))


### Transforming Data
# When assumptions do seem to be violated, transformations of one or
# both variables may remove the problem. Consider the data set 
# "xydata.csv", which we will investigate and possibly transform.

     xydata = read.csv(file.choose(),sep=",", header=T)

# Make sure that the lmtest, nortest, and car packages are installed 
# and loaded, so that we can test the assumptions; use "search()".

## Diagnostic Plots
# We first construct a model and plot the data:
     xy.model=lm(xydata$y~xydata$x)
     plot(xy.model)				# click on the frame for the figure
# Fig, 1 (Residuals vs Fitted Values) suggests heteroscedasticity,
# and there are several outlying residuals. The qq plot (fig. 2) suggests
#  non-normality; note the departures from the qq line.
# Nonstability of the variances (heteroscedasticity) is also suggested
# by fig. 4 (standardized residuals vs leverage).
# Finally, if we plot Cook's distance, cases 10 and 12 stand out:
      plot(xy.model,which=4)

     dev.off()

## Tests of Assumptions
    harvtest(xy.model)		# linearity; p = .746
    ad.test(xydata$y)		# normality; p - .004
    ncvTest(xy.model)		# homoscedasticity; p = .050

# Although the linearity test does not indicate a violation of that assumption,
# the test results provide evidence of nonnormality and heteroskedasticity.

# To deal with the problems,we consider an approach to transforming
# data. The Box-Cox procedure finds a value of lambda that is the basis for a
# power transformation. We first obtain the optimal value of lambda: 
      boxcox(xy.model)
# It appears that the optimal lambda is greater than 2. We can do better:
     boxcox(xy.model,lambda=seq(2,4,1/10)) # explores region from 2 to 4
                                          # in increments of 1/10
# Log-lilihood appears to be maximized at lambda = 2.5. or 2.6
     dev.off()

# Now to use the lambda value:
     lambda = 2.5
    gm =exp(mean(log(xydata$y))) 			# geometric mean
     y.t =(xydata$y^lambda - 1)/(lambda * gm^(lambda-1)) #transformed scores

# Let's examine the transformed data
    model.y.t=lm(y.t~xydata$x)
# Testing assumptions
     harvtest(model.y.t)		# linearity; p = .980
	ad.test(y.t)			# normality; p = .401
     ncvTest(model.y.t)		# homoscedasticity; p = .905    
#  None of the three test results approach significance. Box-Cox
# transformations will not always be successful but often do improve
# matters, as in this example.

## NOTE: If the optimal lambda = 0, the transformation is gm*log(y)
# where gm is defined as above.

# Clean up 
	rm(gm,lambda,model.y.t,xy.model,xydata,y.t)


### Robust estimation
# There are various ways to change the weights of data points so that
# larger residuals have relatively less effect on estimates of the
# coefficients. Huber's M- estimation procedure is one widely
# ecommended approach. For this, we need the "MASS" package
	library(MASS)

# We use the Royer data to compare results of the linear and robust linear
# model analyses.
	attach(submult)
	mult.lm=lm(multrt~multacc)	# Standardmodel
	mult.rlm=rlm(multrt~multacc)	# Robust linear model
	summary(mult.lm)
     summary(mult.rlm)

# For the OLS model, we have
#	            Estimate Std. Error t value     
#	(Intercept) 10.35458    1.90377   5.439 
#	multacc     -0.06734    0.02366  -2.846   
#	Residual standard error: 2.229 on 26 degrees of freedom

#  For the RLM (robust linear) model, we have
#	            Value   Std. Error t value
#	(Intercept) 14.2239  1.3501    10.5356
#	multacc     -0.1149  0.0168    -6.8475
#	Residual standard error: 1.478 on 26 degrees of freedom

# The estimates of the coefficients clearly have  changed. They also
# have smaller standard errors. Most importantly, the residual SE is
# noticeably reduced, indicating an improved fit of the model to the data.

# NOTE: Robust methods do not address violations of assumptions but
# only reduce the contribution of extreme residuals in the estimation process.

##  Exercise 20.4
# Apply the M-estimation method to the xydata (both the original and 
# the transformed); discuss the relationship of the results to the
# earlier OLS analyses.

	detach(submult)
	rm(mult.lm,mult.rlm)

############################################################

#####              Script for Session 21
#####		Introduction to Multiple Regression

### Data Exploration
# We begin by examining total cholesterol (tc) in women as a function of 
#  age and basal metabolism (bmi). For convenience, we first create a data
#  frame for those women under 65 who have scores on all three variables:
	data.w=na.omit(subset(seasons,sex==1&age<=65,select=c(age,tc,bmi)))  

## Plotting the Data
# Let's plot the data 
     pairs(data.w,					# sets up a matrix of scatterplots
	panel = function(x,y){
 	   points(x,y)
	   abline(lm(y~x),col="red",lwd=2,lty=2)  		# OLS line   
	   lines(lowess(x,y),col="darkgreen",lwd=2)},  	# smoother 
	diag.panel = function(x){		# places histograms on diagonal	
	par(new=TRUE)								# keep old graphs
	hist(x,main="",axes=FALSE)}
	)

# The BMI histogram indicates a pronounced skew. Also, the BMI~Age, and
#   BMI~TC scatterplots (bottom row) appear to have outliers as do the 
# tc~age and tc~bmi plots (middle row). In summary, it may be useful
# to delete outliers or to transform the data.
      dev.off()

## Summary Statistics
# Some statistics will supplement our impressions based on the plots
#  and provide a basis for comparison with alternative analyses:
	summary(data.w)
# Note fbat for the bmi data max - 3rdQu is more than 6 times greater
# than 1st Qu. - min, reinforcing our sense of extreme skew.
# Also, get the standard errors of the means. 
     N =  nrow(data.w); N			# 190 cases
# Given N, we can calculate the standard errors
     se1=round(sapply(data.w,sd)/sqrt(N),3); se1

##Exercise 21.1
# Get kurtosis and skewness values (kurt.data, skew.data) for data.w. decimal
# Round to 2 places. Make a data frame that includes summary(data.w), and 
# the stamdard errors, skew, and kurtosis values of age. tc, and bmi.
# Also, test the skewness (skew1) and kurtosis (kurt1) values. See Session 2 
# for examples. Don't forget to first load the "moments" package. 
# Call this table  sum1.data. We will refer to it again, shortly.

	se1 =round(sapply(data.w,sd)/sqrt(N),2)		
     skew1=round(skewness(data.w),2)
     kurt1 =round(kurtosis(data.w),2)
     sum1.data=rbind(summary(data.w),se1,skew1,kurt1)


## A Closer Look at the tc-bmi Relationship
     attach(data.w)
     par(fig=c(0,0.8,0,0.8)) 		# set up for new figure with
      						# room for boxplot outside
     plot(tc~bmi,xlab="BMI",ylab = "TC",pch=19)
     lines(lowess(bmi,tc), lwd=2,col="red") 
     abline(reg=coef(lm(tc~bmi)),lwd=2)

 # Now to add boxplots    
    par(fig=c(0,0.8,.45,.9), new=TRUE)
    boxplot(bmi,horizontal=TRUE, axes=FALSE)
    par(fig=c(0.55,.92,0,0.8),new=TRUE)
    boxplot(tc, axes=FALSE) 
# The boxplots indicate skewed distributions with several outliers.

	dev.off()
 
	sort(bmi,decreasing=T)  		# To find outlying values of bmi
# We remove the 9 cases that have outlying BMI values (Those >= 40).
     data.w2 = subset(data.w,bmi<40)
     N2 = dim(data.w2)[1]

## Summary Statistics after Deleting Outliers
	library(moments)
#Then,
	se2 =round(sapply(data.w2,sd)/sqrt(N2),2)		
     skew2=round(skewness(data.w2),2)
     kurt2 =round(kurtosis(data.w2),2)
     sum2.data=rbind(summary(data.w2),se2,skew2,kurt2)

## Comparisons of Statistics for the Original and Reduced Data
	sum1.data				# From Exercise 21.1
	sum2.data
# The major differences are in the bmi results. The distribution
# is less skewed as indicated by the reduced ratio, 
#   max-3rdQu)/(1st Qu. - min), and the reduced skewness,
#  standard error and kurtosis values of the bmi scores. 

#  Now let's compare the correlation and regression statistics
#  The "psych" library provides the correlations and p values:
	library(psych)
      cor1=corr.test(data.w)
      cor2=corr.test(data.w2)
# To see the correlations
	round(cbind(cor1$r,cor2$r),3)
# The 1st 3 columns are for data.w, next 3 are data.w2
# The age-bmi correlation is roughly 3 times greater in the
#  data.w2 set and the tc-bmi correlation is about 50% greater.
# Eliminating the bmi extreme values indicates a stronger relationship
# of tc and age with bmi for the smaller set of  bmi values.

# The p values are
	round(cbind(cor1$p,cor2$p),3)
# tc is significantly correlated with age and bmi in both analyses. 

# Clean up, retaining data.w and data.w2:
	rm(cor1,cor2,kurt1,kurt2,N,N2,skew1,skew2,se1,se2,sum1.data,sum2.data)
 

### Regression Statistics
## One vs Two Predictors
# Look at the dependence of tc on age and on bmi in the second model
     summary(lm(data.w2$tc~data.w2$age))
	summary(lm(data.w2$tc~data.w2$bmi))
# and, regressing on both predictors,
     model.w2 = lm(data.w2$tc~data.w2$bmi+data.w2$age)
	summary(model.w2)

# Note the differences between the comparable coefficients for the one-
# predictor and the two-predictor regression. In particular, the slope of tc
# as a function of bmi is lower (as is its std. error) when adjusted for
# age (model.w2). The respective slopes are 1.4739 and 1.9340.

## Exercise 21.2
# Check the homoscedasticity and normality assumptions for model.w2.
#  Is there a problem? (Don't forget to load the necessary packages)

## Exercise 21.3  
# Use the robust regression function on tc~age+bmi. Is there any evidence
#  that this provides a better fit to the data?

## Interpreting Multiple R-Squared
     tcfit.mr=fitted(model.w2)	# Get the predicted values
     cor(tcfit.mr,data.w2$tc)^2	# Squared correlation of predicted
							# and observed values
# Compare the preceding correlation with that reported in
#   "summary(model.w2)"

## Diagnosing Collinearity
# When spredictors are highly correlated, they are redundant; the standard
# error of coefficients are increased as a result of collinearity and,
# consequently, the estimate of the coefficien and the power of tests
# are decreased. The variance inflation factor (VIF) ndicates how much
# the estimated variance of the jth regression coefficient is increaased
# above what it would be if X_i were uncorrelated with the otrher 
# predictors (as in a balanced experimental design). VIF > 10 is often
# suggested as an indicator of collinearity [but see O'Brien (2007,
# Quality & Quantity, 673-690]. 
# Make sure the car library is loaded.

	vif(model.w2)	#  VIF = 1.013729; collinearlity is not a problem.

## NOTE: the lm.ridge fuction may provide better parameter estimates 
#   when collinearity is suspected; package is "MASS".

## Exercise 21.4
# The formula for VIF for predictor X_j is 1/(1-(R_j)^2) where 1/(1-(R_j)^2)
# is the squared correlation of X_j with the other predictors. Use this formula
# to verify the VIF of 1.0137 obtained for the W2 data. Why would the VIF
# be 1 for an orthogonal set of predictors (as in a balanced experimental 
# design)?


### Partitioning the Regression Sum of Squares
# summary.lm provides the coefficients and t tests of the hypothesis that
# the population values equal zero. The output does not depend on the
# order of predictors. However, compare  the output for these two statements
# that include the same predictors, but in different orders.
	summary(aov(data.w2$tc~data.w2$bmi+data.w2$age))
	summary(aov(data.w2$tc~data.w2$age+data.w2$bmi))
# The results differ because the variables are not orthogonal. 

## Exercise 21.5
# The outputs from the 2 aov functions differ. Note that F= 53.613 is the 
# square of the t for age in the summary.lm output, but 13.004 is not 
# the square of the t for  bmi. Conversely, in the second aov o but utput, 
#F = 7.4499 is the square of the t for bmi in the lm output but 59.1673
# is not the square of the t for age in the lm output. What are the two 
# (i.e., lm and aov) functions testing?


### Adjusted R-Squared
# Summary.lm includes "adjusted R-squared: 0.2642." The formula for this is
	R_sq = .2723			# From the lm summary
     p = 2				# no. of predictors
     N = 181				# no. of cases	
	R_sq_adj = 1-(1-R_sq)*(N-1)/(N-1-p);  R_sq_adj
#  The result is the same within .0001
      
# Remove some objects that are no longer needed
     rm(tcfit.mr,R_sq,p,N,R_sq_adj)

##############################################################

#####              Script for Session 22
#####		Inference in Multiple Regression

### Testing Hypotheses about Regression Coefficients
## Testing  Null Hypotheses 
	summary(model.w2)
# Adjusting for age, BMI contributes significantly to the variation in tc;
# (t=2.729); also, adjusting for bmi, age makes a significant contirbution
# (t=7.322). Also, multiple R-squared differs significantly from zero; 
# ("F-statistic: 33.31 on 2 and 178 DF,  p-value: 5.153e-13")

#  To get confidence intervals for the coefficients:
	confint(model.w2)		

#  The output gives the bounds on the intercept and the linear regression
#  coefficients, adjusted for the effect of the other variable: 
#	                 2.5 %     97.5 %
#	(Intercept) 58.8500814 125.634086
#	data.w2$bmi  0.4082652   2.539456		# tc~bmi slope
#	data.w2$age  1.2688493   2.205116		# tc~age slope

# Alternatively, we could calculate these bounds from the summary output; 
# e.g., the upper bound for the intercept is
	92.2421+qt(.025,178,lower.tail=F)*16.9212     # = 125.6341

## Exercise 22.1
# Using the submult data frame, construct a model predicting multrt 
# as a function of subrt and multacc. Then, test the intercept and
# regression coefficients against zero. Also decide whether the
#  multiplr R differs significantly from zero. Finally, find the confience 
# limits on the three parameters.

## Model Comparisons (Partial-F Tests)
# model.w2 includes both bmi and age as predictors of tc. We can
# compare this with a model in which age is the only preictor of tc.
	model.age=lm(data.w2$tc~data.w2$age)
# Does adding bmi to the regression equation significantly improve the fit?
	anova(model.w2,model.age)
# p-value for F is .007; adding bmi to the equation significantly improves
# the fit. Similar tests can be conducted when one model excludes more
# than one predictor from the larger model.

# NOTE: The F for the model comparison was 7.4499. Its square root, 2.729,
# is the t reported in the summary of model.w2 for the test of the slope
# of tc as a function of bmi, adjusted for age. The partial F test
# of the contribution of bmi beond that of age is equivalent to a test
# of the the regression of tc on bmi, adjusted for age.


### Predicting an Individual's Score
# Let's use the data from the women (bmi < 40)
	attach(data.w2)
	y = data.frame(cbind(age=35,bmi=30)) # We predict the tc score for a
							 	# woman with those age & bmi values
	model=lm(tc~age+bmi)			 # a model for the new data frame
	predict(model,y,interval="prediction")  
# The prediction and confidence interval are:
#       fit      lwr      upr
#   197.2523 130.3199 264.1847

## Exercise 22.2
# Find the tc prediction and interval for a man,age = 50 and bmi = 42


### Confidence Intervals for R_Squared (R2) Statistics
## Interval for R2
# The "Functions.R" script contains a function for doing this.
# The function is R_sq.ci(R2,p,n,1-confidence); R2=squared R, n=no. of cases,
# p=number of predictors. For the tc.w data, we have
	R_sq.ci(.2723,2,181,.05)		
## The output is 
##         upper     lower
## [1,] 0.3802179 0.1643821      # The 95% CI bounds for R-squared

## NOTE: A more precise CI is provided by Steiger and Fouladi
## (1992, Behavior Research Methods, Instruments & Computers, 4, 581-582)
## and is instrumented in the R packages MBESS and gsl. Assuming these
## are installed,
	library(MBESS)
	library(gsl)
#  The form of the function is ci.RG(R2,df.1-p,df.2=n-p-1,conf.level); e.g.,
     ci.R2(.2723,2,178,.95)
#  yields .380 and .161.

## Interval for the Difference between Independent R2 Statistics
#  The Functions.R script also contains a function for a confidence interval
# for the difference between two independent R-squares; e.g., men and women.
# See Olkin & Finn (1995,Psychological Bulletin,118,155-164; but also Algina
# & Keselman, 1999, Psychological Methods, 4,76-83). 

#  The function is R_sq_d.R2(R2.1, R2.2,n1,n2,alfa). From our previous model
#  summary (for the women), R1.1 = .2723 and n1 = 181. 

# For the men,
	data0.m=subset(seasons,sex==0&age<=65,select=c(age,tc,bmi))   
      data.m = na.omit(data0.m)
      model.m2 = lm(data.m$tc~data.m$bmi+data.m$age)
	summary(model.m2)
#  We get R2.2 = .0475, and n2=183, Therefore, 
     R_sq_d.R2(.2723,.0475,181,183,.05)
#  The result is
#         upper       lower
# [1,] 0.350736 0.09886403
# We have .95 confidence that the difference between the two R2's lies
#  between .10 and .35. Because this interval does not contains zero, we 
#  reject the null hypothesis of no difference between the squared R's.

## NOTE: Our functions assume large N. Algina & Keselman (see citation above)
# provide tables of minimal N to provide adequate bounds for the difference CI.

## Exercise 22.3
## (a) Using the submult data frame, regress multrt on subrt and multacc. Test 
# whether R2 (R-squared) differs significantly from zero.
# (b) Do the R2 values for boys and girls differ significantly from each other?


### Power Calculations in Multiple Regression
# You should have the "pwr" package installed (see Session 8). Load it:
	library(pwr)

##  A General R Function to Calculate Power 
#  pwr.f2.test(u = NULL, v = NULL, f2 = NULL, sig.level = 0.05, power = NULL)
# All but one of the parameter values must be supplied. So, if we want to
# select the N needed for power to test a certain effect size, we would supply
#  all but the denominator df; u and v are the numerator and denominator df, 
# and f2 is the effect size whose formulation depends on our purpose.

## To solve for N, we first obtain the value of v, using the "pwr.f2.test
# function. Following Cohen (1988), we define Nstar= u + v + 1
# where u and v are the numerator and denominator df.
# Because v = N-p-1,once we know v, we then can obtain N from 
#	 Nstar = u+(N-p-1)+1 = N-p+u which yields
#	 N = Nstar+p-u   (p is the number of predictors)
#  Now let us consider some specific examples .
 
## Case 1: Testing H0: rho2 = 0 
# (rho2 is the squared population correlation)
# This tests whether all p linear regression coeffients = 0
# Assune  that we desire .9 power to reject H0 if rho2 >= .2, and that
#  p = 4. We want to know the required N. Cohen's f2 is defined as
#  f2 = rho2/(1-rho2); in our example, f2 = .2/.8 = .25. Therefore,
     pwr.f2.test(u=4,f2=.25,sig.level=.05,power=.9)

#  The output gives us
#      v = 61.55727 which we round to 62
#   Then Nstar = u+v+1 = 4+62+1=67 and
#	N = Nstar+p-u = 67+4-4=67. 	# NOTE: In Case 1, N = Nstar.

## Case 2: Testing H0: Adding a predictor does not change rho2
#	f2 = hypothesized change/(1-rho2 after predictor is added)
# Assume we want to detect a change from .25 to .3 when a predictor is added
# to a set of 3; desired power = .8. How large should N be?
#	f2 = .05/(1-.3) = .071
	pwr.f2.test(u=1,f2=.071,sig.level=.05,power=.8)
# which yields  v = 110.5007, or 111
#  Then Nstar = = u+v+1 =1+111+1 = 113, and
#       N = Nstar+p-u = 113+4-1 = 116

## Case 3: Testing H0: Adding k predictors does not change rho2
#  Assume we want to detect a change from .44 to .5 when three predictors are 
#  added to a set of 2; desired power = .8. How large should N be?
#	f2 = .06/(1-.5) = .12
	pwr.f2.test(u=3,f2=.12,sig.level=.05,power=.8)
#	 v = 90.85444, or 91
#  Then Nstar = u+v+1 = 3+91+1 = 95, and
#       N = Nstar+p-u = 95+5-3 = 97

## Exercuse 22.4
# Assume a study with three predictors and N = 50 resulted in R = .35. (a) If
# the true population rho = .35, what power do you estimate the study had?
# (b) What N would be needed to have power = .8 to test the hypothesis
#  that rho = .35?


### Checking Assumptions
# As we saw in Session 20, several plots are available to check assumptions
# We may view four plots that relate to the normality and homoscedasticity
#  assumptions, as well as pinpoint certain cases that may affect our
#  inferences. For the tc data for women between the ages of 40 and 65,
	op = par(mfrow=c(2,2))	#To view all 4 plots in the same screen
	plot(model.w2)		# Plot the model ccomponents

# The top left plot is of the standardized residuals as a function of the fittted 
@ (predicted) tc values. It is not clear whether there is heteroscedasticity 
# However, the plot does have several points (cases 67, 172, 179) that may be 
# outliers. At the least these cases deserve further examination.

# The "car" library provides a test of the homoscedasticity assumption:
	library(car)
	ncvTest(model.w2)
# p = .026. This may be due to the three points labeled in the plot.

# Most points in the Q-Q plot fall on the line. We can test to see if the
# departure from normality at the end points is sufficient to reject
# the assumption of normality. 
# Load the nortest library, which was installed for Session 20.
	library(nortest)
	ad.test(residuals(model.w2))	# The p-value = .43; the departure from
					# normality does not appear to be significant.
	dev.off()

## Exercise 22.5
# Working with model.w2 (women's tc scores ~ age + bmi, uses the 
#  influencePlot (id.method="identify") and the plot (...which=4) functions
#  to determine the three points that appear to have the most effect on parameter 
#  estimates. Create a new data frame, excluding those cases. How do the
# results compare withe those for the original data.w2 data frame?

## NOTE: Refer to Session 18 for an example of an influence plot, and to
# Session 19 for an example of the plot(model) function.
# These require that the car library be loaded. 

# Clean up
	detach(data.w2)
	rm(data0.m,model,model.age,model.m2,model.w2,op,y)

### Selecting Variables for Best Prediction
# We first create a data frame consisting of several variables.
	attach(seasons)
	stats.df = na.omit(data.frame(cbind(beck_d, height,weight,
		bmi,tc,beck_a)))  # keep only cases with all measures
	detach(seasons)
	attach(stats.df)
# We now construct a model and get information on it
	stats.lm = lm(beck_d~height+weight+bmi+tc+beck_a)
	summary(stats.lm)

# The results of the t tests suggest that beck_a (aggression score)
# strongly predicts beck_d (depression score). Linear coefficients
# for the other predictors do not differ significantly from zero.
	
#  Can we predict better with fewer variables?
	summary(step(stats.lm))
			
# The default is backward elimination in which the process starts
# with the full set of predictors, dropping the one correspoonding
# to the smallest contribution, and continuing in that way.
# R's step function uses the AIC criterion; tc is the first
# variable dropped from the model followed by height and then weight. 

# In viewing the results for the full and reduced models, there are
# several noteworthy changes: (1) The reduced model has only two
# of the original five variables. (2) bmi now makes a significant
# contribution. (3) Estimates for the intercept and the regression
# coefficient for bmi are clearly changed. (4) Despite dropping three 
# predictors, the adjusted R2 values are very similar. 

	detach(stats.df)
	rm(stats.df,stats.lm)

## Exercise 22.6
# Using the Royer.data (if not still in your workspace, see Session 18),
# create a new data frame for all those cases having the six variables:
# for addition, subtraction, and  multiplication --  accuracy and
# response time. Then form a model for multrt as a function of the
# other five variables. 
# (a) Get summaries of this model's and of the step function's results
# What differences are there between the two sets of results? 
# (b) What other changes, if any, are present if "grade" is added to the
# stepwise regression?

## Exercise 22.7
# Stepwise regression may capitalize on chance and the resulting
# R2 may be spuriously high. Cross-validation provides a better way
# of assessing the model. Calculate the regression parameters based
# on only the first 151 cases in stats.df; use the two-parameter
# model (beck_d~bmi, beck_a). Predict the scores of the remaining 
# 152 cases. Then correlate the predicted and observed scores for 
# the second set. How does the squared correlation compare to the
# adjusted R2 in the final stepwise result we previously obtained?

##############################################################
####              Script for Session 23
		Additional Topics in Multiple Regression
## In this session we extend multtple regresion anaalyses to po;ynomial
#  and interaction terms.

### Polynomial Regression: Continuous Predictors
## The Basic Model
# In Session 21, we created the data.w data frame for women under 65
#  who had both tc and bmi scores We use those data here. For our purposes,
#  we divide data.w into two data sets:
	data.wa =data.w[1:95,]
	data.wb =data.w[96:190,]
# using the a data set, we regress tc on bmi and on bmi squared:
     bmi2a=data.wa$bmi^2
	wa.lm=lm(tc~bmi+bmi2a,data=data.wa)
	summary(wa.lm)
# The result tells us that the slope of the tc~bmi differs significantly
# from zero. There is also a (nonsignificant) suggestion that the tc~bmi
# function has a quadratic component.

## Exercise 23.1
# Find the variance inflation factors for the wa.lm. You will need
# the car library.

# The answer to Exercise 23.1 suggests multicolinearity (vif>10)
# Multicollinearity, large correlations between predictors, is a problem 
#  when the predictors are x and X-squared. This often results in large 
#  stadard errors and therefore less precise prediction. One solution
#  is centering, creating a variable by subtracting its mean from all values.

## The Centered Model
	bmica=data.wa$bmi-mean(data.wa$bmi)	# centering bmi
     bmi2ca=bmica^2
	wac.lm=lm(tc~bmica+bmi2ca,data=data.wa)
	summary(wac.lm)

## Comparing the Results for the Two Models
#  From the summary for the uncentered regression analtsis we have

#Coefficients:
#           Estimate Std. Error t value Pr(>|t|)  
#(Intercept) 85.43993   52.69049   1.622   0.1083  
#bmi          7.27444    3.31243   2.196   0.0306 *
#bmi2a       -0.09179    0.04890  -1.877   0.0637 .
#Residual standard error: 37.57 on 92 degrees of freedom
#Multiple R-squared: 0.07498,    Adjusted R-squared: 0.05487 
#F-statistic: 3.729 on 2 and 92 DF,  p-value: 0.02773

# And from the centered summary, we have

#Coefficients:
#             Estimate Std. Error t value Pr(>|t|)    
#(Intercept) 214.64763    4.41322  48.637  < 2e-16 ***
#bmica         2.33987    0.85890   2.724  0.00771 ** 
#bmi2ca       -0.09179    0.04890  -1.877  0.06367 .
#Residual standard error: 37.57 on 92 degrees of freedom
#Multiple R-squared: 0.07498,    Adjusted R-squared: 0.05487 
#F-statistic: 3.729 on 2 and 92 DF,  p-value: 0.02773 43

# Note the reduction in the standard errors (and p values)
# of the intercept and linear when bmi is centered.

## Exercise 23.2
# Find the vif for the wac.lm model. Is there evidence that 
# multicolinearity has been reduced?

#  Note that the indices of fit are identical (e.g., R-squared, F,the
#  residual sum of squares) but the parameters and their standard
#  errors differ, as well as the p-values of the parameters. If the fits 
#  are the same, why center? One answer can be found in using the
#  parameters from the two data.wa analyses to predict to the data.wb data.

#  We first predict the scores for data.wb using the parameters from the
#  uncentered analysis of data.wa.
	tcb1_hat  = 85.43993+7.27444*data.wb$bmi-0.09179*data.wb$bmi^2
# Now predict the data.wb scores using the centered parameters.
	bmicb=data.wb$bmi-mean(data.wb$bmi)
	bmi2cb=bmicb^2
	tcb2_hat=214.64763+2.33987*bmicb-0.09179*bmi2cb
# Now to compare the two fits.
	sd(tcb1_hat-data.wb$bmi)	#SD(predicted - observed), uncentered
	sd(tcb2_hat-data.wb$bmi)	#SD(predicted - observed), centered
#  The standardeviation is less for the parameters derived from the centered
#  analysis, indicating a closer fit.

## Exercise 23.3
# Use the two sets of parameters from an analysis of the wb.data to predict
#  the wa.data.

## Exercise 23.4
#  Using the entire data.w set (190 cases), do the regression analysis with
# both centered an uncentered variables, including an x-cubed term.

# Clean up the workspace.
	rm(bmi2a, bmi2ca,bmi2cb,bmica, bmicb,tcb1_hat,tcb2_hat,wa.lm,wac.lm)


### Trend Analysis Revisited 
# When the predictor has equally spaced levels with equal numbers of 
#  observations, polynomial coefficients are readily available in both
#  published tables and statistical software, as well as by using the
#  R functions, such as contr.poly. However, regression analysis provides a
#  more general approach, able to deal with unequal spacing and unequal n. 

#  We use the adddacc data for first through fifth grade from the
#  Royer.data. If this data frame is no longer in your workspace, read
#  "Royer Data.cvs".
	addacc.df=subset(Royer.data,grade<6,select=c(grade,addacc))

#  The lm function provides the least-squares regression equation in which 
#  the contribution of each variable is adjusted for those of the other
#  variables. However, we wish to test the variability contributed
#  by each po;ynomial coomonent, adjusted only ror the compoents of
# lower order. Thus, we first calculate the linear sum of squares, then the
#  quadratic adjusted for the linear contribution, thewn the cubic 
#  adjusted for the linear and quadratic, and so on.
#  The aov function does this for us.
	g2 = addacc.df$grade^2
	g3=addacc.df$grade^3
	g4=addacc.df$grade^4
	summary(aov(addacc~grade+g2+g3+g4,data=addacc.df))
# Only the linear and quadratic terms are significant. This is consistent
#  with a plot of the adddacc means as a function of grades.

##  Exercise 23.5
#  Following the script in Session 9, plot means and error bars for addacc
#  as a function of grade.
	
## NOTE: The approach taken in Sessions 11/12 in which we obtained
# polynomial coefficients using the "contr.poly" function, gives slightly 
# different results because it assumes the grades have the same
# numbers of cases, which they do not.


### Interaction Terms in Multiple Regression 
#  "data.w2" contains tc,age, and bmi scores for women in the seasons study 
# who are no older than 65 and have bmi <= 40. "data.m2", with the same
#  constraints, can be obtained from "data.m". We will do so, then form
#  a single data frame that will be used in an analysis of the effects of sex,
#  age, and their interaction on tc. 
	data.m2=subset(data.m, data.m$bmi<=40)
	sex=c(rep(0,nrow(data.m2)),rep(1,nrow(data.w2)))
	tc.mw = c(data.m2$tc,data.w2$tc)
	bmi.mw = c(data.m2$bmi,data.w2$bmi)
	age.mw = c(data.m2$age,data.w2$age)

# The combined data frame is
	data.mw = data.frame(cbind(sex,tc.mw,bmi.mw,age.mw))
	head(data.mw)

## Plotting the Data
# We first plot tc~age for each sex
	plot(data.w2$tc~data.w2$age, pch=19,xlab="AGE",
		ylab="Total Cholesterol")
	abline(lm(data.w2$tc~data.w2$age),lwd=3)
	points(data.m2$age,data.m2$tc,pch=22,col="red")
	abline(lm(data.m2$tc~data.m2$age),col="red",lwd=3)
	legend("topleft",c("Women","Men"), pch=c(19,22), cex=.8)
# There appears to be a wider range of scores for the women. Most striking
# is the apparent interaction between sex and age. The red least-squares line
# for men is relatively flat whereas the leaast-squares line for women 
# indicates an increase in tc with age. We may see this analytically:
	 summary(lm(data.m2$tc~data.m2$age))	# analysis for men
	 summary(lm(data.w2$tc~data.w2$age))	# Analysis for women
# The slopes are -4.661e-04 (men) and 1.8123 (women). The regression of
# tc on age is significant only for women.

	dev.off()

## Analyzing Interaction Between Continuous and Dichotomous Predictors
	summary(lm(tc.mw~age.mw*sex))
#  The interaction is quite significant, indicating that the tc~age functions
#  have different slopes for men and women.

##  Interactions Between Two Ccontinuous Predictors
# In Session 21, we regressed tc for women (ages 20 to 65, bmi <=40 )
#  on age and bmi. We now consider whether these two variables interact.
	summary(lm(tc~age*bmi, data = data.w2))
#  Note that * in the summary function represents main effects + interaction
# The results indicate that all three terms contribute to the population
# variance of the tc scores. In particular, the significant interaction
# tells us that the slope of tc as a function of one variable differs
# for different values of the other vcariable. 

##  Exercise 23.6
# Using the data.m2 data frame,find the regression equation for tc~age+
#  bmi. Repeat the process, now including the interaction term. Finally,
# use the anova function to test the relative fits of the two models.

###  Centering, Again (pp.592-593)
# First find the variance inflation factors (the car library must be loaded)
	vif(lm(tc~age*bmi, data = data.w2))
# The vifs for all three terms sre well above the often recommended  
# value of 10, probably because the interaction tends to be highly  
# correlated with the other predictors. We will redo the analysis,
#  after first centering all the variables. 
	attach(data.w2)
	tc.c=tc-mean(tc)
	age.c=age-mean(age)
	bmi.c=bmi-mean(bmi)
	summary(lm(tc.c~age.c*bmi.c))
#  The interaction slope (.135) is unchanged by centering but correlations
#  among terms are greatly reduced, as are standard errors of estimate,
#  indicating improved estimation. The variance inflation factors are now
#  considerably lower than previously as seen by
	vif(lm(tc.c~age.c*bmi.c))
	
## Exercise 23.7
# Center the variables for the data.w2 analysis and compare results,
#  including vifs, with those for the analysis in Exercise 23.4.

# Clean up
	detach(data.w2)
	rm(addacc.df,age.c,age.mw,bmi.c, bmi.mw,data.m,data.m2, data.w,
	data.mw,data.w2,data.wa,data.wb,g2,g3,g4,sex,tc.c,tc.mw)


############################################################

#####			Script for Session 24
#####		Regression with Categorical Predictors

# In Session 23, we analyzed a data set involving a contninuous amd a
# dichotomous variable. We now extend this to include categorical 
# predictors that may have more than two levels. The first step is to code
# such categorical variables; that is to assign numerical values to their
# levels so that the variables can be included in regression equaltions.


### A Simple Example: Regression with a Dichotomous Predictor (p. 604)
# Consider a standard example where the factor A has two levels.
	y = c(4,8,7,9,7,2,5,3)
	A = c(rep("A1",5),rep("A2",3))
	my.data = data.frame(cbind(A,y)); my.data
# We can do an analysis of variance (or a t test):
	summary(aov(y~A,data=my.data))

# The result is:
#             Df Sum Sq Mean Sq F value  Pr(>F)  
# A           1 25.208 25.2083  8.1027 0.02932 *
# Residuals   6 18.667  3.1111

# Linear regression does the analysis, yielding the same F and R-squared. 
# Call the predictor (A) x and
	x=c(rep(1,5),rep(0,3))   # Any two numbers will give the same result
	summary(lm(my.data$y~x))	
# The result is
#             Estimate Std. Error t value Pr(>|t|)  
# (Intercept)    3.333      1.018   3.273   0.0170 *
# x              3.667      1.288   2.847   0.0293 *
# -
# Residual standard error: 1.764 on 6 degrees of freedom
# Multiple R-squared: 0.5745,     Adjusted R-squared: 0.5036 
# statistic: 8.103 on 1 and 6 DF,  p-value: 0.02932 

# NOTE: The slope for y~x = 3.667 = difference between the group means.
# The relationship beteen the ANOVA and the regression analysis is:
# the t for x (2.847) = sqrt(F) where F = 8.103). Lastly,
# R-squared = SSA/(SSA+SSresiduals) = .5745  

# For the two-level independent variable, there is no advantage to the
# regression approach. However, with multifactor designs, particularly
# when cell sizes vary, it is often helpful to view the analysis as a
# regression problem. 


### Analyses in Unbalanced (Non-orthogonal) Designs
#  We use data from the "unbalanced data.csv" file:
	unbal.df=read.csv(file.choose(),header=T)
	head(unbal.df)

## Method 1 (Type III SS;  the default in SPSS)
# Adjusts each term for all main and interaction effects
	A=factor(unbal.df$A)
	C=factor(unbal.df$C)
	model1=aov(unbal.df$Y~A*C,contrasts=list(A=contr.sum, 
		C=contr.sum))
	drop1(model1, .~., test="F") 
#Estimates of effects are:
	coefficients(model1)
# Note that all estimates are unweighted means of all cell means that
# are involved in the source, and effect estimates are based on subtraction 
# from the intercept (the unweighted mean of all 9 cell means.

##  Method II (Type II SS, default in lm function)
#  Adjusts each term for all effects of same or lower order
#  This can be obtained from the "car" library
	library(car)
	model2=lm(unbal.df$Y~A*C,type="II")
	Anova(model2)
	coefficients(model2)
# Here the intercept estimate is the mean of the A1C1 cell and all
# other effect estimates are differences between cell means and the intercept

## NOTE: the "lm" function does have a Type = "III" option but the results 
# are not what we want. Judging by requests for help on the internet,
# this has been a source of considerable confusion.

##  Method III (Type I SS; hierarchical)
#  A hierarchy with A calculated without adjustment:
	summary(aov(unbal.df$Y~A+C+A:C))  
# The A is unadjusted but the C is adjusted; 
#  If we want both A and C unadjusted, reverse the order of A and C:
	summary(aov(unbal.df$Y~C+A+C:A)) 	# C is now unadjusted
#  We now have SS, F, p values for all terms with A and C unadjusted.

# clean up
	rm(A,C,model1,model2, my.data,unbal.df,x,y)

## Exercise 24.1
# From the seasons data, create a data frame with beck_d, mstatus, and 
# schoolyr. Include only cases with all three measures. (see Session 22 for
# an example). Then apply methods 1 and 2 to beck_d as a function of mstatus
# and schoolyr.


### Interactions of Categorical and Continuous Variables 
# We wish to test whether age and educational level (el) interact
# in their effects on total chosterol level (tc); specifically, 
# is the increase in tc with age a function of educational 
# level? We begin by reading the data file "ed level data.csv".
	tc.el=read.csv(file.choose(),header=T)
	head(tc.el)
	el=factor(tc.el$el)

## Plotting the Best-Fit Linear Regression Lines 
	EL1=subset(tc.el,el==1,select=c(age,tc))
	fit1=lm(EL1$tc~EL1$age)
	plot(EL1,col="red",ylim=c(100,400),pch=NA) # "pch=NA" => no points
	abline(fit1,col="red",lty=1, lwd=2)

	EL2=subset(tc.el,el==2,select=c(age,tc))
	fit2=lm(EL2$tc~EL2$age)
	abline(fit2,col="blue", lty=2, lwd=2)

	EL3=subset(tc.el,el==3,select=c(age,tc))
	fit3=lm(EL3$tc~EL3$age)
	abline(fit3,col="darkgreen",lty=3, lwd=2)

# One of the three slopes (the blue EL2 line) is not parallel to the others.
# Whether this signifies a significant interaction requires further analysis. 
	dev.off()

## Testing the Interaction  
	model.el=aov(tc.el$tc~tc.el$age*el,contrasts=list(el=contr.sum))
	drop1(model.el, .~., test="F")
# The only significant effect is age; the slopes for tc ~ age do not
# differ significantly as a function of el.

# We might construct a model without the interaction, and test whether
# the absence of the interaction harms the fit.
	model.el2=aov(tc.el$tc~el+tc.el$age,contrasts=list(el=contr.sum))
	drop1(model.el2, .~., test="F")
	anova(model.el,model.el2)
# The F (=.641) indicates no significant difference between the two models.
# Therefore, the parsimonious approach is to omit the interaction when
# fitting the data.

## Exercise 24.2
# In Session 18, we created the "submult" data frame. Plot "multrt" as a 
# function of "multacc" for grades 3 - 5, then test whether the rate 
# of change in the RT-Accuracy relationship depends on grade level.

	rm(el,EL1,EL2,EL3,fit1,fit2,fit3, model.el,model.el2,tc.el,submult, 
		Royer.data,seasons)

############################################################

#####			Script for Session 25
#####			Analysis of Covariance 

# In analyzing total cholesterol (tc) as a function of educational
# level (el) and age in Session 24, our primary interest was in the 
# interaction of age and el. In the analysis of covariance, our assumption
# is that there is no interaction, and our focus is on the test of the 
# categorical variable. The role of the covariate is to reduce error 
# variance by taking advantage of the relation between the dependent
# variable (e.g., tc) and the covariate (e.g., age). In this session,
# we first consider a one-factor data set, a study of test 
# scores (Y) as a function of instructional method in learning probability.
# Read the file "pl.csv".
	pl.data = read.csv(file.choose(),header=T)
	head(pl.data)		# I="Instructional Method)" 
					#(Standard, Low Explanation,High Explanation);
					# Y = test score;X = aptitude score
	attach(pl.data)
# In obtaining adjusted means, to avoid errors,
	i.f = factor(I)


### ANOVA for a One-Factor Design
# We first do a standard one-factor ANOVA for later comparison with ANCOVA;
	summary(aov(Y~i.f))
# The effects of I are not significant at the .05 level (p=.062)

### ANCOVA for a One-Factor Design
## Using the aov Function
# We now carry out the ANCOVA for these data:
	pl.aov=aov(Y~X+i.f)
	summary(pl.aov)
# The I factor is now significant; p= .010.

## Using the lm Function
# instead of the aov function:
	pl.lm=lm(Y~X+i.f)
	anova(pl.lm)

# For additional information,
	summary(pl.lm)
# This contrasts the LE and S adjusted means against the intercept
# (the adjusted HE mean), as well as an R-s1quare and 
# related statistics for the multiple regression of Y on x and I.

## NOTE: The order of variables is critical in these functions. We want 
# to test I after X's contribution to the variance has been removed.


### Adjusting Group Means
# We need the "effects" library
	library(effects)
	effect("i.f",pl.lm)
# The adjusted means are:
#       HE        LE         S 
# 0.5342383 0.3800132 0.4189360 

## Exercise 25.1 
# To directly verify the above results, we need the within-group regression
# coefficient (or "slope"), b_S/A. 
# (a) To get b_S/A, caclulate for each of the three groups,
# sum(X-mean(X))*(Y-mean(Y)) and add these. Then divide by
# the sum of the three within-groups SS_X, where SS_X=sum((X-mean(X))^2).
# The aggregate function will be helpful here. Your result should be .00732.
# (b) The adjusted means for group j is 
# mean(Y_j)-b_S/A*(mean(X_j)-grand mean(X)). Calculate this for all
# three conditions. You should get the results from the effect function.
 

### Tests After ANCOVA
## The Tukey HSD Test
# The test uses the covariance-adjusted residual error.
	library(multcomp)
	TukeyHSD(pl.aov)
# The result includes confidence bounds and shows that only the LE and HE
# means differ signidicantly. For comparispn purposes, 
	pl.aov2=aov(Y~i.f)		#covariate omitted
	TukeyHSD(pl.aov2)		# no difference is significant

## More Complex Contrasts
# We contrast the HE adjusted mean against the average of the S and LE means,
# and the difference between the SE and LE means. There are several ways to
# perform these tests as discussed in Session 10; the main difference is that 
# we now use the adjusted means and adjusted residual error.
	library(gmodels)
	w = rbind(c(1,-.5,-.5),c(0,1,-1))
	fit.contrast(pl.lm,"i.f",w,conf.int=.95, df=TRUE)
# p values and confidence limits indicate that the HE mean differs
# from the average of the S and LE means, and that the latter two do
# not differ.

	
### Evaluating Assumptions Underlying ANCOVA
# Not an assumption but useful to check the correlation between X and Y.
# The higher r, the greater efficiency and if the correlation is very
# low, ANCOVA may be counterproductive, particularly with small N.
# In our example,
	cor.test(X,Y)	# r(X,Y)=.548 and is very significant

## A.1 Normality of the Distribution of the Residuals.
	resY=residuals(pl.lm)
	qqnorm(resY,main = "Probability Learning Residuals")
	qqline(resY)	# There is no obvious depareture from the line
	dev.off()

## A.2 Homoscedasticity of the Residuals
	plot(resY~predict.lm(pl.lm))
	abline(lm(resY~predict.lm(pl.lm)),col="red",lwd=2,lty=2)
# Although there is considerable variability,there is no clear evidence
# of a systematic pattern indicating heteroscedasticity, However, we can
# test the possibility: First turn off the plot.
	dev.off()
#E Now the test
	library(lmtest)
	bptest(pl.lm)		#The Breusch-Pagan test
# p=.758; the test provides no evidence of heteroscedasticity

## A.3 Homogeneity of the Group Residual Variances
	library(car)
	leveneTest(resY,i.f)	
# p=.58; there is no evidence of heterogeneity of variance

# Violation of the preceding assumptions typically hace little effect.
# More important are the following:

## A.4 Linearity. We test for a quadratic component.
	X2 = X^2
	pl.aov.sq=aov(Y~X+X2+I)
	summary(pl.aov.sq)
# The squared X contributes to a significant result for I. We can compare 
# the original covariance model and this one:
	anova(pl.aov,pl.aov.sq)
# p=.004; we reject the linear model for one including a quadratic term.
# The inclusion of an X-squared term improves the power of the test in
# this example.

## NOTE: The preceding is an example of nonlinear analysis of covariance.


## A.5 Homogeneity of Regression Slopes
# This is nasically an assumption of no interaction between I and X. Therefore,
	summary(aov(Y~i.f*X))
# The interaction p-value is  0.71325; clearly not significant
# NOTE: The "i.f*X" notation removes I variability before X, In ANCOVA, X
# should be indicated before I in the model. However, order does not
# affect the interaction (I:X) results.

## A.6 Independence of Treatment and Covariate
# Are the X means affected by the treatment?
	summary(aov(X~i.f))
# p = .412 indicating independence of X and the treatment, I.

## Exercise 25.2
# The submult data frame based on the Royer arithmetic accuracy and response time
# data provides us with several covariates. If not still in your workspace,
# read the "Royer Data.csv" file and recreate the data frame:
	Royer.data = read.csv(file.choose(),header=T,sep=",")
	submult=na.omit(subset(Royer.data,grade==3,select=1:6))
# (a) Ignoring gender,analyze the effects of grade on multrt using
# multacc as a covariate. Make sure grade has been designated as a factor.
# (b) Compare hte results with those for a model without the covariate.
# (c) get the adjusted means for each grade.
# (d) Test the assumptions of ANCOVA


### More than One Covariate
# Create an additional covariate, U.
	set.seed(1357)
	U=round(40+60*Y+rnorm(48,0,10))
# U has the following characteristics
	c(min(U),max(U))	# = [1] 31 89
	cor(Y,U)			# = 0.6793295
	cor(X,U)			# = 0.3389517
# Reanalyze the probability learning data
	pl.aov3=aov(Y~U+X+i.f)
	summary(pl.aov3)
# In this case, the p-value for I is considerably higher than with only X
# as a covariate, perhaps because of the overlap between U and X; their
# correlation is significantly higher than zero.

## Exercise 25.3
# Using the submult data frame (from Exercise 25.2), redo the test of grade
# effects on multrt, using both subrt and multacc as covariates. Compare the
# results with those in Exercise 25.2 using only multacc.

# Clean up  the workspace
	detach(pl.data)
	rm(i.f,pl.aov,pl.aov2,pl.aov3, pl.aov.sq,pl.data, pl.lm,resY,U,w,
	X2, submult,Royer.data,seasons)
		

### ANCOVA for Factorial Designs
# In Session 9, we tested the effects of instructions ("instruct") AND
# format on inference verification (ivt) in the study by Wiley and Voss.
# We mow add sentence verification (svt) scores as a covariate. The 
# correlation is .264, not high, but we will see if svt contributes to the
# power of the analysis.

## Significance Tests
# If W_V.data is no longer in your workspace (use ls() or LS() to check),
# read in the wiley.csv data.
#  W_V.data= read.csv(file.choose(),header=T,sep=",")
	attach(W_V.data)
	I.f=factor(instruct)
	F.f=factor(format)
# First do the ANOVA of the inferenc verification (ivt) data
	wv.aov1=aov(ivt~I.f*F.f)
	summary(wv.aov1)
# Now for the ANCOVA with sentene verification (sct) as a covaariate.
	wv.aov2=aov(ivt~svt+I.f*F.f)
	summary(wv.aov2)
# Now compare the models
	anova(wv.aov1,wv.aov2)
# The residual sums of squares (RSS) are about 10% less for the ANCOVA.
# Looking at the two summaries, our conclusions have not been affected;
# format is significant in both and instruct in neither. However, the p
# values have been reduced and the significant reduction in RSS indicates
# that confidence intervals will be narrower when the covariate is included.

## Adjusted Means
# The "effects" package has a function for adjusting all means
# However, only lm and glm models will usually work
	wv.lm=lm(ivt~svt+I.f*F.f)	
	summary(wv.lm)

#  If the Effects package is installed and loaded into the workspace,
	allEffects(wv.lm)[2]		# Yiwelds the adjusted cell means:

#	 I.f*F.f effect
#	   F.f
#	I.f        1        2
#	  1 68.83561 76.47873
#	  2 70.08561 74.79200,
#	  3 69.58868 75.77848
#	  4 74.18205 89.00883
# The rows are the instructional levels (Narrative, Summary, Explanation,
# Argument); the coluns are the formats (Text, Web)

# We can plot these adjusted means, as well as the ivt~svt relationship
	plot(allEffects(wv.lm),ask=F)	  #"ask=T" (the default) causes R to ask
							  # which plot(ivt~svt or effects) you want
# ivt clearly increases as svt does.Also, the Ff:2 (web Format) has higher
# ivt adjusted means than does the Ff:1 (Text) Format. Finally, the Argument
# adjusted mean in the Web condition is markedly higher than the other adjusted 
# means.

	dev.off()

# Clean up
	detach(W_V.data)
	rm(F.f,I.f,W_V.data,wv.aov1, wv.aov2,wv.lm)


